@@ -19,6 +19,8 @@ authenticated against.  This defaults to `login`
 
     auth.pam.service: login
 
+.. note:: Solaris-like (SmartOS, OmniOS, ...) systems may need ``auth.pam.service`` set to ``other``.
+
 .. note:: PAM authentication will not work for the ``root`` user.
 
     The Python interface to PAM does not support authenticating as ``root``.
@@ -43,15 +45,23 @@ from ctypes.util import find_library
 from salt.utils import get_group_list
 from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
 
-LIBC = CDLL(find_library('c'))
+# Import 3rd-party libs
+import salt.ext.six as six
 
-CALLOC = LIBC.calloc
-CALLOC.restype = c_void_p
-CALLOC.argtypes = [c_uint, c_uint]
+try:
+    LIBC = CDLL(find_library('c'))
 
-STRDUP = LIBC.strdup
-STRDUP.argstypes = [c_char_p]
-STRDUP.restype = POINTER(c_char)  # NOT c_char_p !!!!
+    CALLOC = LIBC.calloc
+    CALLOC.restype = c_void_p
+    CALLOC.argtypes = [c_uint, c_uint]
+
+    STRDUP = LIBC.strdup
+    STRDUP.argstypes = [c_char_p]
+    STRDUP.restype = POINTER(c_char)  # NOT c_char_p !!!!
+except AttributeError:
+    HAS_LIBC = False
+else:
+    HAS_LIBC = True
 
 # Various constants
 PAM_PROMPT_ECHO_OFF = 1
@@ -142,7 +152,7 @@ def __virtual__():
     '''
     Only load on Linux systems
     '''
-    return HAS_PAM
+    return HAS_LIBC and HAS_PAM
 
 
 def authenticate(username, password):
@@ -156,6 +166,13 @@ def authenticate(username, password):
     '''
     service = __opts__.get('auth.pam.service', 'login')
 
+    if isinstance(username, six.text_type):
+        username = username.encode(__salt_system_encoding__)
+    if isinstance(password, six.text_type):
+        password = password.encode(__salt_system_encoding__)
+    if isinstance(service, six.text_type):
+        service = service.encode(__salt_system_encoding__)
+
     @CONV_FUNC
     def my_conv(n_messages, messages, p_response, app_data):
         '''
@@ -167,7 +184,7 @@ def authenticate(username, password):
         p_response[0] = cast(addr, POINTER(PamResponse))
         for i in range(n_messages):
             if messages[i].contents.msg_style == PAM_PROMPT_ECHO_OFF:
-                pw_copy = STRDUP(str(password))
+                pw_copy = STRDUP(password)
                 p_response.contents[i].resp = cast(pw_copy, c_char_p)
                 p_response.contents[i].resp_retcode = 0
         return 0
