@@ -24,6 +24,7 @@ import salt.exceptions
 import salt.minion
 import salt.utils
 import salt.utils.event
+import salt.utils.files
 import salt.utils.kinds
 
 # pylint: disable=import-error,no-name-in-module,redefined-builtin
@@ -496,7 +497,7 @@ class Key(object):
                     if minion not in minions and minion not in preserve_minions:
                         shutil.rmtree(os.path.join(m_cache, minion))
             cache = salt.cache.factory(self.opts)
-            clist = cache.ls(self.ACC)
+            clist = cache.list(self.ACC)
             if clist:
                 for minion in clist:
                     if minion not in minions and minion not in preserve_minions:
@@ -974,7 +975,7 @@ class RaetKey(Key):
                     if minion not in minions and minion not in preserve_minions:
                         shutil.rmtree(os.path.join(m_cache, minion))
                 cache = salt.cache.factory(self.opts)
-                clist = cache.ls(self.ACC)
+                clist = cache.list(self.ACC)
                 if clist:
                     for minion in clist:
                         if minion not in minions and minion not in preserve_minions:
@@ -1017,7 +1018,7 @@ class RaetKey(Key):
         '''
         Use libnacl to generate and safely save a private key
         '''
-        import libnacl.public
+        import libnacl.dual  # pylint: disable=import-error,3rd-party-module-not-gated
         d_key = libnacl.dual.DualSecret()
         keydir, keyname, _, _ = self._get_key_attrs(keydir, keyname,
                                                     keysize, user)
@@ -1411,14 +1412,13 @@ class RaetKey(Key):
         keydata = {'priv': priv,
                    'sign': sign}
         path = os.path.join(self.opts['pki_dir'], 'local.key')
-        c_umask = os.umask(191)
-        if os.path.exists(path):
-            #mode = os.stat(path).st_mode
-            os.chmod(path, stat.S_IWUSR | stat.S_IRUSR)
-        with salt.utils.fopen(path, 'w+') as fp_:
-            fp_.write(self.serial.dumps(keydata))
-            os.chmod(path, stat.S_IRUSR)
-        os.umask(c_umask)
+        with salt.utils.files.set_umask(0o277):
+            if os.path.exists(path):
+                #mode = os.stat(path).st_mode
+                os.chmod(path, stat.S_IWUSR | stat.S_IRUSR)
+            with salt.utils.fopen(path, 'w+') as fp_:
+                fp_.write(self.serial.dumps(keydata))
+                os.chmod(path, stat.S_IRUSR)
 
     def delete_local(self):
         '''
