@@ -8,6 +8,7 @@ from __future__ import absolute_import
 import contextlib  # For < 2.7 compat
 import copy
 import errno
+import glob
 import logging
 import os
 import re
@@ -21,6 +22,9 @@ try:
 except ImportError:
     from pipes import quote as _quote
 
+# Import third party libs
+import salt.ext.six as six
+from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module
 try:
     import rarfile
     HAS_RARFILE = True
@@ -29,10 +33,10 @@ except ImportError:
 
 # Import salt libs
 from salt.exceptions import SaltInvocationError, CommandExecutionError
-from salt.ext.six import string_types, integer_types
-from salt.ext.six.moves.urllib.parse import urlparse as _urlparse  # pylint: disable=no-name-in-module
 import salt.utils
+import salt.utils.files
 import salt.utils.itertools
+import salt.utils.templates
 
 # TODO: Check that the passed arguments are correct
 
@@ -154,6 +158,9 @@ def list_(name,
             salt '*' archive.list ftp://10.1.2.3/foo.rar
     '''
     def _list_tar(name, cached, decompress_cmd, failhard=False):
+        '''
+        List the contents of a tar archive.
+        '''
         dirs = []
         files = []
         links = []
@@ -193,7 +200,7 @@ def list_(name,
                         raise CommandExecutionError('Invalid CLI options')
                 else:
                     if salt.utils.which('xz') \
-                            and __salt__['cmd.retcode'](['xz', '-l', cached],
+                            and __salt__['cmd.retcode'](['xz', '-t', cached],
                                                         python_shell=False,
                                                         ignore_retcode=True) == 0:
                         decompress_cmd = 'xz --decompress --stdout'
@@ -216,6 +223,7 @@ def list_(name,
 
     def _list_zip(name, cached):
         '''
+        List the contents of a zip archive.
         Password-protected ZIP archives can still be listed by zipfile, so
         there is no reason to invoke the unzip command.
         '''
@@ -257,6 +265,9 @@ def list_(name,
             raise CommandExecutionError('{0} is not a ZIP file'.format(name))
 
     def _list_rar(name, cached):
+        '''
+        List the contents of a rar archive.
+        '''
         dirs = []
         files = []
         if HAS_RARFILE:
@@ -313,6 +324,9 @@ def list_(name,
         path = parsed.path or parsed.netloc
 
         def _unsupported_format(archive_format):
+            '''
+            Raise the proper exception message for the given archive format.
+            '''
             if archive_format is None:
                 raise CommandExecutionError(
                     'Unable to guess archive format, please pass an '
@@ -401,6 +415,36 @@ def list_(name,
         raise CommandExecutionError(exc.error, info=info)
 
 
+_glob_wildcards = re.compile('[*?[]')
+
+
+def _glob(pathname):
+    '''
+    In case ``pathname`` contains glob wildcards, performs expansion and returns
+    the possibly empty list of matching pathnames. Otherwise returns a list that
+    contains only ``pathname`` itself.
+    '''
+    if _glob_wildcards.search(pathname) is None:
+        return [pathname]
+    else:
+        return glob.glob(pathname)
+
+
+def _expand_sources(sources):
+    '''
+    Expands a user-provided specification of source files into a list of paths.
+    '''
+    if sources is None:
+        return []
+    if isinstance(sources, six.string_types):
+        sources = [x.strip() for x in sources.split(',')]
+    elif isinstance(sources, (float, six.integer_types)):
+        sources = [str(sources)]
+    return [path
+            for source in sources
+            for path in _glob(source)]
+
+
 @salt.utils.decorators.which('tar')
 def tar(options, tarfile, sources=None, dest=None,
         cwd=None, template=None, runas=None):
@@ -437,6 +481,9 @@ def tar(options, tarfile, sources=None, dest=None,
         Comma delimited list of files to **pack** into the tarfile. Can also be
         passed as a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     dest
         The destination directory into which to **unpack** the tarfile
 
@@ -459,6 +506,8 @@ def tar(options, tarfile, sources=None, dest=None,
 
         # Create a tarfile
         salt '*' archive.tar cjvf /tmp/tarfile.tar.bz2 /tmp/file_1,/tmp/file_2
+        # Create a tarfile using globbing (2017.7.0 and later)
+        salt '*' archive.tar cjvf /tmp/tarfile.tar.bz2 '/tmp/file_*'
         # Unpack a tarfile
         salt '*' archive.tar xf foo.tar dest=/target/directory
     '''
@@ -468,20 +517,14 @@ def tar(options, tarfile, sources=None, dest=None,
         # should at least let them know of their silliness.
         raise SaltInvocationError('Tar options can not be empty')
 
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
-
     cmd = ['tar']
-    if dest:
-        cmd.extend(['-C', '{0}'.format(dest)])
-
     if options:
         cmd.extend(options.split())
 
     cmd.extend(['{0}'.format(tarfile)])
-
-    if sources is not None:
-        cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
+    if dest:
+        cmd.extend(['-C', '{0}'.format(dest)])
 
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
@@ -589,6 +632,9 @@ def cmd_zip(zip_file, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the zip file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     template : None
         Can be set to 'jinja' or another supported template engine to render
         the command arguments before execution:
@@ -623,12 +669,12 @@ def cmd_zip(zip_file, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.cmd_zip /tmp/zipfile.zip /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.cmd_zip /tmp/zipfile.zip '/tmp/sourcefile*'
     '''
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
     cmd = ['zip', '-r']
     cmd.append('{0}'.format(zip_file))
-    cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
                                template=template,
@@ -655,6 +701,9 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the zip file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     template : None
         Can be set to 'jinja' or another supported template engine to render
         the command arguments before execution:
@@ -685,6 +734,8 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.zip /tmp/zipfile.zip /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.zip /tmp/zipfile.zip '/tmp/sourcefile*'
     '''
     if runas:
         euid = os.geteuid()
@@ -696,11 +747,7 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
             )
 
     zip_file, sources = _render_filenames(zip_file, sources, None, template)
-
-    if isinstance(sources, string_types):
-        sources = [x.strip() for x in sources.split(',')]
-    elif isinstance(sources, (float, integer_types)):
-        sources = [str(sources)]
+    sources = _expand_sources(sources)
 
     if not cwd:
         for src in sources:
@@ -709,13 +756,12 @@ def zip_(zip_file, sources, template=None, cwd=None, runas=None):
                     'Relative paths require the \'cwd\' parameter'
                 )
     else:
-        def _bad_cwd():
-            raise SaltInvocationError('cwd must be absolute')
+        err_msg = 'cwd must be absolute'
         try:
             if not os.path.isabs(cwd):
-                _bad_cwd()
+                raise SaltInvocationError(err_msg)
         except AttributeError:
-            _bad_cwd()
+            raise SaltInvocationError(err_msg)
 
     if runas and (euid != uinfo['uid'] or egid != uinfo['gid']):
         # Change the egid first, as changing it after the euid will fail
@@ -851,9 +897,9 @@ def cmd_unzip(zip_file,
 
         salt '*' archive.cmd_unzip /tmp/zipfile.zip /home/strongbad/ excludes=file_1,file_2
     '''
-    if isinstance(excludes, string_types):
+    if isinstance(excludes, six.string_types):
         excludes = [x.strip() for x in excludes.split(',')]
-    elif isinstance(excludes, (float, integer_types)):
+    elif isinstance(excludes, (float, six.integer_types)):
         excludes = [str(excludes)]
 
     cmd = ['unzip']
@@ -989,7 +1035,6 @@ def unzip(zip_file,
         os.seteuid(uinfo['uid'])
 
     try:
-        exc = None
         # Define cleaned_files here so that an exception will not prevent this
         # variable from being defined and cause a NameError in the return
         # statement at the end of the function.
@@ -997,9 +1042,9 @@ def unzip(zip_file,
         with contextlib.closing(zipfile.ZipFile(zip_file, "r")) as zfile:
             files = zfile.namelist()
 
-            if isinstance(excludes, string_types):
+            if isinstance(excludes, six.string_types):
                 excludes = [x.strip() for x in excludes.split(',')]
-            elif isinstance(excludes, (float, integer_types)):
+            elif isinstance(excludes, (float, six.integer_types)):
                 excludes = [str(excludes)]
 
             cleaned_files.extend([x for x in files if x not in excludes])
@@ -1016,26 +1061,26 @@ def unzip(zip_file,
                     if extract_perms:
                         perm = zfile.getinfo(target).external_attr >> 16
                         if perm == 0:
-                            umask_ = os.umask(0)
-                            os.umask(umask_)
+                            umask_ = salt.utils.files.get_umask()
                             if target.endswith('/'):
                                 perm = 0o777 & ~umask_
                             else:
                                 perm = 0o666 & ~umask_
                         os.chmod(os.path.join(dest, target), perm)
     except Exception as exc:
-        pass
+        if runas:
+            os.seteuid(euid)
+            os.setegid(egid)
+        # Wait to raise the exception until euid/egid are restored to avoid
+        # permission errors in writing to minion log.
+        raise CommandExecutionError(
+            'Exception encountered unpacking zipfile: {0}'.format(exc)
+        )
     finally:
         # Restore the euid/egid
         if runas:
             os.seteuid(euid)
             os.setegid(egid)
-        if exc is not None:
-            # Wait to raise the exception until euid/egid are restored to avoid
-            # permission errors in writing to minion log.
-            raise CommandExecutionError(
-                'Exception encountered unpacking zipfile: {0}'.format(exc)
-            )
 
     return _trim_files(cleaned_files, trim_output)
 
@@ -1117,6 +1162,9 @@ def rar(rarfile, sources, template=None, cwd=None, runas=None):
         Comma-separated list of sources to include in the rar file. Sources can
         also be passed in a Python list.
 
+        .. versionchanged:: 2017.7.0
+            Globbing is now supported for this argument
+
     cwd : None
         Run the rar command from the specified directory. Use this argument
         along with relative file paths to create rar files which do not
@@ -1139,11 +1187,11 @@ def rar(rarfile, sources, template=None, cwd=None, runas=None):
     .. code-block:: bash
 
         salt '*' archive.rar /tmp/rarfile.rar /tmp/sourcefile1,/tmp/sourcefile2
+        # Globbing for sources (2017.7.0 and later)
+        salt '*' archive.rar /tmp/rarfile.rar '/tmp/sourcefile*'
     '''
-    if isinstance(sources, string_types):
-        sources = [s.strip() for s in sources.split(',')]
     cmd = ['rar', 'a', '-idp', '{0}'.format(rarfile)]
-    cmd.extend(sources)
+    cmd.extend(_expand_sources(sources))
     return __salt__['cmd.run'](cmd,
                                cwd=cwd,
                                template=template,
@@ -1183,7 +1231,7 @@ def unrar(rarfile, dest, excludes=None, template=None, runas=None, trim_output=F
         salt '*' archive.unrar /tmp/rarfile.rar /home/strongbad/ excludes=file_1,file_2
 
     '''
-    if isinstance(excludes, string_types):
+    if isinstance(excludes, six.string_types):
         excludes = [entry.strip() for entry in excludes.split(',')]
 
     cmd = [salt.utils.which_bin(('unrar', 'rar')),
@@ -1193,9 +1241,9 @@ def unrar(rarfile, dest, excludes=None, template=None, runas=None, trim_output=F
             cmd.extend(['-x', '{0}'.format(exclude)])
     cmd.append('{0}'.format(dest))
     files = __salt__['cmd.run'](cmd,
-                               template=template,
-                               runas=runas,
-                               python_shell=False).splitlines()
+                                template=template,
+                                runas=runas,
+                                python_shell=False).splitlines()
 
     return _trim_files(files, trim_output)
 
@@ -1229,7 +1277,7 @@ def _render_filenames(filenames, zip_file, saltenv, template):
         temp file, rendering that file, and returning the result.
         '''
         # write out path to temp file
-        tmp_path_fn = salt.utils.mkstemp()
+        tmp_path_fn = salt.utils.files.mkstemp()
         with salt.utils.fopen(tmp_path_fn, 'w+') as fp_:
             fp_.write(contents)
         data = salt.utils.templates.TEMPLATE_REGISTRY[template](
@@ -1254,7 +1302,9 @@ def _render_filenames(filenames, zip_file, saltenv, template):
 
 
 def _trim_files(files, trim_output):
-    # Trim the file list for output
+    '''
+    Trim the file list for output.
+    '''
     count = 100
     if not isinstance(trim_output, bool):
         count = trim_output
