@@ -1,19 +1,20 @@
-# # -*- coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 '''
-Manage a GPG keychains, add keys, create keys, retrieve keys
-from keyservers.  Sign, encrypt and sign & encrypt text and files.
+Manage a GPG keychains, add keys, create keys, retrieve keys from keyservers.
+Sign, encrypt and sign plus encrypt text and files.
 
 .. versionadded:: 2015.5.0
 
 .. note::
-    The ``python-gnupg`` library and gpg binary are
-    required to be installed.
+
+    The ``python-gnupg`` library and ``gpg`` binary are required to be
+    installed.
 
 '''
 
 # Import python libs
 from __future__ import absolute_import
-import distutils.version  # pylint: disable=import-error,no-name-in-module
+import functools
 import logging
 import os
 import re
@@ -23,17 +24,10 @@ import time
 import salt.utils
 import salt.syspaths
 from salt.exceptions import SaltInvocationError
+from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import 3rd-party libs
 import salt.ext.six as six
-try:
-    from shlex import quote as _cmd_quote  # pylint: disable=E0611
-except ImportError:
-    from pipes import quote as _cmd_quote
-
-from salt.exceptions import (
-    SaltInvocationError
-)
 
 # Set up logging
 log = logging.getLogger(__name__)
@@ -77,19 +71,18 @@ VERIFY_TRUST_LEVELS = {
     '4': 'Ultimate'
 }
 
-HAS_LIBS = False
 GPG_1_3_1 = False
-
 try:
     import gnupg
-    HAS_LIBS = True
+    HAS_GPG_BINDINGS = True
+    GPG_1_3_1 = _LooseVersion(gnupg.__version__) >= _LooseVersion('1.3.1')
 except ImportError:
-    pass
+    HAS_GPG_BINDINGS = False
 
 
-def _check_gpg():
+def _gpg():
     '''
-    Looks to see if gpg binary is present on the system.
+    Returns the path to the gpg binary
     '''
     # Get the path to the gpg binary.
     return salt.utils.which('gpg')
@@ -99,41 +92,99 @@ def __virtual__():
     '''
     Makes sure that python-gnupg and gpg are available.
     '''
-    if _check_gpg() and HAS_LIBS:
-        gnupg_version = distutils.version.LooseVersion(gnupg.__version__)
-        if gnupg_version >= '1.3.1':
-            global GPG_1_3_1
-            GPG_1_3_1 = True
-        return __virtualname__
-    return False
+    if not _gpg():
+        return (False, 'The gpg execution module cannot be loaded: '
+                'gpg binary is not in the path.')
+
+    return __virtualname__ if HAS_GPG_BINDINGS \
+        else (False, 'The gpg execution module cannot be loaded; the '
+                     'gnupg python module is not installed.')
+
+
+def _get_user_info(user=None):
+    '''
+    Wrapper for user.info Salt function
+    '''
+    if not user:
+        # Get user Salt runnining as
+        user = __salt__['config.option']('user')
+
+    userinfo = __salt__['user.info'](user)
+
+    if not userinfo:
+        if user == 'salt':
+            # Special case with `salt` user:
+            # if it doesn't exist then fall back to user Salt running as
+            userinfo = _get_user_info()
+        else:
+            raise SaltInvocationError('User {0} does not exist'.format(user))
+
+    return userinfo
+
+
+def _get_user_gnupghome(user):
+    '''
+    Return default GnuPG home directory path for a user
+    '''
+    if user == 'salt':
+        gnupghome = os.path.join(salt.syspaths.CONFIG_DIR, 'gpgkeys')
+    else:
+        gnupghome = os.path.join(_get_user_info(user)['home'], '.gnupg')
+
+    return gnupghome
+
+
+def _restore_ownership(func):
+    @functools.wraps(func)
+    def func_wrapper(*args, **kwargs):
+        '''
+        Wrap gpg function calls to fix permissions
+        '''
+        user = kwargs.get('user')
+        gnupghome = kwargs.get('gnupghome')
+
+        if not gnupghome:
+            gnupghome = _get_user_gnupghome(user)
+
+        userinfo = _get_user_info(user)
+        run_user = _get_user_info()
+
+        if userinfo['uid'] != run_user['uid'] and os.path.exists(gnupghome):
+            # Given user is different from one who runs Salt process,
+            # need to fix ownership permissions for GnuPG home dir
+            group = __salt__['file.gid_to_group'](run_user['gid'])
+            for path in [gnupghome] + __salt__['file.find'](gnupghome):
+                __salt__['file.chown'](path, run_user['name'], group)
+
+        # Filter special kwargs
+        for key in list(kwargs):
+            if key.startswith('__'):
+                del kwargs[key]
+
+        ret = func(*args, **kwargs)
+
+        if userinfo['uid'] != run_user['uid']:
+            group = __salt__['file.gid_to_group'](userinfo['gid'])
+            for path in [gnupghome] + __salt__['file.find'](gnupghome):
+                __salt__['file.chown'](path, user, group)
+
+        return ret
+
+    return func_wrapper
 
 
 def _create_gpg(user=None, gnupghome=None):
     '''
     Create the GPG object
     '''
-    if not user:
-        user = __salt__['config.option']('user')
-
-    if user == 'salt':
-        if gnupghome:
-            homeDir = gnupghome
-        else:
-            homeDir = os.path.join(salt.syspaths.CONFIG_DIR, 'gpgkeys')
-    else:
-        if gnupghome:
-            homeDir = gnupghome
-        else:
-            userinfo = __salt__['user.info'](user)
-            if userinfo:
-                homeDir = '{0}/.gnupg'.format(userinfo['home'])
-            else:
-                raise SaltInvocationError('User does not exist')
+    if not gnupghome:
+        gnupghome = _get_user_gnupghome(user)
 
     if GPG_1_3_1:
-        gpg = gnupg.GPG(homedir='{0}'.format(homeDir))
+        gpg = gnupg.GPG(homedir=gnupghome)
     else:
-        gpg = gnupg.GPG(gnupghome='{0}'.format(homeDir))
+        gpg = gnupg.GPG(gnupghome=gnupghome)
+
     return gpg
 
 
@@ -166,11 +217,12 @@ def search_keys(text, keyserver=None, user=None):
         Text to search the keyserver for, e.g. email address, keyID or fingerprint.
 
     keyserver
-        Keyserver to use for searching for GPG keys, defaults to pgp.mit.edu
+        Keyserver to use for searching for GPG keys, defaults to pgp.mit.edu.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     CLI Example:
 
@@ -191,18 +243,20 @@ def search_keys(text, keyserver=None, user=None):
 
         _keys = []
         for _key in _search_keys(text, keyserver, user):
-            tmp = {}
-            tmp['keyid'] = _key['keyid']
-            tmp['uids'] = _key['uids']
-            if 'expires' in _key:
-                if _key['expires']:
-                    tmp['expires'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['expires'])))
-            if 'date' in _key:
-                if _key['date']:
-                    tmp['created'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['date'])))
-            if 'length' in _key:
+            tmp = {'keyid': _key['keyid'],
+                   'uids': _key['uids']}
+
+            expires = _key.get('expires', None)
+            date = _key.get('date', None)
+            length = _key.get('length', None)
+
+            if expires:
+                tmp['expires'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['expires'])))
+            if date:
+                tmp['created'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['date'])))
+            if length:
                 tmp['keyLength'] = _key['length']
             _keys.append(tmp)
         return _keys
@@ -213,11 +267,12 @@ def list_keys(user=None, gnupghome=None):
     List keys in GPG keychain
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -228,26 +283,28 @@ def list_keys(user=None, gnupghome=None):
     '''
     _keys = []
     for _key in _list_keys(user, gnupghome):
-        tmp = {}
-        tmp['keyid'] = _key['keyid']
-        tmp['fingerprint'] = _key['fingerprint']
-        tmp['uids'] = _key['uids']
-        if 'expires' in _key:
-            if _key['expires']:
-                tmp['expires'] = time.strftime('%Y-%m-%d',
-                                               time.localtime(float(_key['expires'])))
-        if 'date' in _key:
-            if _key['date']:
-                tmp['created'] = time.strftime('%Y-%m-%d',
-                                               time.localtime(float(_key['date'])))
-        if 'length' in _key:
+        tmp = {'keyid': _key['keyid'],
+               'fingerprint': _key['fingerprint'],
+               'uids': _key['uids']}
+
+        expires = _key.get('expires', None)
+        date = _key.get('date', None)
+        length = _key.get('length', None)
+        owner_trust = _key.get('ownertrust', None)
+        trust = _key.get('trust', None)
+
+        if expires:
+            tmp['expires'] = time.strftime('%Y-%m-%d',
+                                           time.localtime(float(_key['expires'])))
+        if date:
+            tmp['created'] = time.strftime('%Y-%m-%d',
+                                           time.localtime(float(_key['date'])))
+        if length:
             tmp['keyLength'] = _key['length']
-        if 'ownertrust' in _key:
-            if _key['ownertrust']:
-                tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
-        if 'trust' in _key:
-            if _key['trust']:
-                tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
+        if owner_trust:
+            tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
+        if trust:
+            tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
         _keys.append(tmp)
     return _keys
 
@@ -257,44 +314,49 @@ def list_secret_keys(user=None, gnupghome=None):
     List secret keys in GPG keychain
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' gpg.list_secret_keys
+
     '''
     _keys = []
     for _key in _list_keys(user, gnupghome, secret=True):
-        tmp = {}
-        tmp['keyid'] = _key['keyid']
-        tmp['fingerprint'] = _key['fingerprint']
-        tmp['uids'] = _key['uids']
-        if 'expires' in _key:
-            if _key['expires']:
-                tmp['expires'] = time.strftime('%Y-%m-%d',
-                                               time.localtime(float(_key['expires'])))
-        if 'date' in _key:
-            if _key['date']:
-                tmp['created'] = time.strftime('%Y-%m-%d',
-                                               time.localtime(float(_key['date'])))
-        if 'length' in _key:
+        tmp = {'keyid': _key['keyid'],
+               'fingerprint': _key['fingerprint'],
+               'uids': _key['uids']}
+
+        expires = _key.get('expires', None)
+        date = _key.get('date', None)
+        length = _key.get('length', None)
+        owner_trust = _key.get('ownertrust', None)
+        trust = _key.get('trust', None)
+
+        if expires:
+            tmp['expires'] = time.strftime('%Y-%m-%d',
+                                           time.localtime(float(_key['expires'])))
+        if date:
+            tmp['created'] = time.strftime('%Y-%m-%d',
+                                           time.localtime(float(_key['date'])))
+        if length:
             tmp['keyLength'] = _key['length']
-        if 'ownertrust' in _key:
-            if _key['ownertrust']:
-                tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
-        if 'trust' in _key:
-            if _key['trust']:
-                tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
+        if owner_trust:
+            tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
+        if trust:
+            tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
         _keys.append(tmp)
     return _keys
 
 
+@_restore_ownership
 def create_key(key_type='RSA',
                key_length=1024,
                name_real='Autogenerated Key',
@@ -350,14 +412,16 @@ def create_key(key_type='RSA',
         an epoch value, or 0 for a non-expiring key.
 
     use_passphrase
-        Whether to use a passphrase with the signing key.  Passphrase is received from pillar.
+        Whether to use a passphrase with the signing key. Passphrase is received
+        from Pillar.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -393,7 +457,7 @@ def create_key(key_type='RSA',
         create_params['expire_date'] = expire_date
 
     if use_passphrase:
-        gpg_passphrase = __salt__['pillar.item']('gpg_passphrase')
+        gpg_passphrase = __salt__['pillar.get']('gpg_passphrase')
         if not gpg_passphrase:
             ret['res'] = False
             ret['message'] = "gpg_passphrase not available in pillar."
@@ -432,11 +496,12 @@ def delete_key(keyid=None,
         Secret keys must be deleted before deleting any corresponding public keys.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -475,10 +540,10 @@ def delete_key(keyid=None,
             ret['res'] = False
             ret['message'] = 'Secret key exists, delete first or pass delete_secret=True.'
             return ret
-        elif skey and delete_secret:
+        elif skey and delete_secret and str(gpg.delete_keys(fingerprint, True)) == 'ok':
             # Delete the secret key
-            if str(gpg.delete_keys(fingerprint, True)) == 'ok':
-                ret['message'] = 'Secret key for {0} deleted\n'.format(fingerprint)
+            ret['message'] = 'Secret key for {0} deleted\n'.format(fingerprint)
+
         # Delete the public key
         if str(gpg.delete_keys(fingerprint)) == 'ok':
             ret['message'] += 'Public key for {0} deleted'.format(fingerprint)
@@ -495,17 +560,18 @@ def get_key(keyid=None, fingerprint=None, user=None, gnupghome=None):
     Get a key from the GPG keychain
 
     keyid
-        The keyid of the key to be retrieved.
+        The key ID (short or long) of the key to be retrieved.
 
     fingerprint
         The fingerprint of the key to be retrieved.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -520,26 +586,31 @@ def get_key(keyid=None, fingerprint=None, user=None, gnupghome=None):
     '''
     tmp = {}
     for _key in _list_keys(user, gnupghome):
-        if _key['fingerprint'] == fingerprint or _key['keyid'] == keyid:
+        if (_key['fingerprint'] == fingerprint or
+                _key['keyid'] == keyid or
+                _key['keyid'][8:] == keyid):
             tmp['keyid'] = _key['keyid']
             tmp['fingerprint'] = _key['fingerprint']
             tmp['uids'] = _key['uids']
-            if 'expires' in _key:
-                if _key['expires']:
-                    tmp['expires'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['expires'])))
-            if 'date' in _key:
-                if _key['date']:
-                    tmp['created'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['date'])))
-            if 'length' in _key:
+
+            expires = _key.get('expires', None)
+            date = _key.get('date', None)
+            length = _key.get('length', None)
+            owner_trust = _key.get('ownertrust', None)
+            trust = _key.get('trust', None)
+
+            if expires:
+                tmp['expires'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['expires'])))
+            if date:
+                tmp['created'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['date'])))
+            if length:
                 tmp['keyLength'] = _key['length']
-            if 'ownertrust' in _key:
-                if _key['ownertrust']:
-                    tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
-            if 'trust' in _key:
-                if _key['trust']:
-                    tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
+            if owner_trust:
+                tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
+            if trust:
+                tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
     if not tmp:
         return False
     else:
@@ -551,17 +622,18 @@ def get_secret_key(keyid=None, fingerprint=None, user=None, gnupghome=None):
     Get a key from the GPG keychain
 
     keyid
-        The keyid of the key to be retrieved.
+        The key ID (short or long) of the key to be retrieved.
 
     fingerprint
         The fingerprint of the key to be retrieved.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -573,55 +645,61 @@ def get_secret_key(keyid=None, fingerprint=None, user=None, gnupghome=None):
 
         salt '*' gpg.get_secret_key keyid=3FAD9F1E user=username
 
-
     '''
     tmp = {}
     for _key in _list_keys(user, gnupghome, secret=True):
-        if _key['fingerprint'] == fingerprint or _key['keyid'] == keyid:
+        if (_key['fingerprint'] == fingerprint or
+                _key['keyid'] == keyid or
+                _key['keyid'][8:] == keyid):
             tmp['keyid'] = _key['keyid']
             tmp['fingerprint'] = _key['fingerprint']
             tmp['uids'] = _key['uids']
-            if 'expires' in _key:
-                if _key['expires']:
-                    tmp['expires'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['expires'])))
-            if 'date' in _key:
-                if _key['date']:
-                    tmp['created'] = time.strftime('%Y-%m-%d',
-                                                   time.localtime(float(_key['date'])))
-            if 'length' in _key:
+
+            expires = _key.get('expires', None)
+            date = _key.get('date', None)
+            length = _key.get('length', None)
+            owner_trust = _key.get('ownertrust', None)
+            trust = _key.get('trust', None)
+
+            if expires:
+                tmp['expires'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['expires'])))
+            if date:
+                tmp['created'] = time.strftime('%Y-%m-%d',
+                                               time.localtime(float(_key['date'])))
+            if length:
                 tmp['keyLength'] = _key['length']
-            if 'ownertrust' in _key:
-                if _key['ownertrust']:
-                    tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
-            if 'trust' in _key:
-                if _key['trust']:
-                    tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
+            if owner_trust:
+                tmp['ownerTrust'] = LETTER_TRUST_DICT[_key['ownertrust']]
+            if trust:
+                tmp['trust'] = LETTER_TRUST_DICT[_key['trust']]
     if not tmp:
         return False
     else:
         return tmp
 
 
-def import_key(user=None,
-               text=None,
+@_restore_ownership
+def import_key(text=None,
                filename=None,
+               user=None,
                gnupghome=None):
     r'''
     Import a key from text or file
 
-    user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
-
     text
         The text containing to import.
 
     filename
         The filename containing the key to import.
 
+    user
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
+
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -632,9 +710,9 @@ def import_key(user=None,
 
     '''
     ret = {
-           'res': True,
-           'message': ''
-          }
+        'res': True,
+        'message': ''
+        }
 
     gpg = _create_gpg(user, gnupghome)
 
@@ -651,11 +729,6 @@ def import_key(user=None,
 
     imported_data = gpg.import_keys(text)
 
-    # include another check for Salt unit tests
-    gnupg_version = distutils.version.LooseVersion(gnupg.__version__)
-    if gnupg_version >= '1.3.1':
-        GPG_1_3_1 = True
-
     if GPG_1_3_1:
         counts = imported_data.counts
         if counts.get('imported') or counts.get('imported_rsa'):
@@ -687,19 +760,21 @@ def export_key(keyids=None, secret=False, user=None, gnupghome=None):
     Export a key from the GPG keychain
 
     keyids
-        The keyid(s) of the key(s) to be exported.  Can be specified as a comma
-        separated string or a list.  Anything which GnuPG itself accepts to
-        identify a key - for example, the keyid or the fingerprint could be used.
+        The key ID(s) of the key(s) to be exported. Can be specified as a comma
+        separated string or a list. Anything which GnuPG itself accepts to
+        identify a key - for example, the key ID or the fingerprint could be
+        used.
 
     secret
-        Export the secret key identified by the keyid information passed.
+        Export the secret key identified by the ``keyids`` information passed.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -709,7 +784,7 @@ def export_key(keyids=None, secret=False, user=None, gnupghome=None):
 
         salt '*' gpg.export_key keyids=3FAD9F1E secret=True
 
-        salt '*' gpg.export_key keyid="['3FAD9F1E','3FBD8F1E']" user=username
+        salt '*' gpg.export_key keyids="['3FAD9F1E','3FBD8F1E']" user=username
 
     '''
     gpg = _create_gpg(user, gnupghome)
@@ -719,6 +794,7 @@ def export_key(keyids=None, secret=False, user=None, gnupghome=None):
     return gpg.export_keys(keyids, secret)
 
 
+@_restore_ownership
 def receive_keys(keyserver=None, keys=None, user=None, gnupghome=None):
     '''
     Receive key(s) from keyserver and add them to keychain
@@ -731,18 +807,23 @@ def receive_keys(keyserver=None, keys=None, user=None, gnupghome=None):
         separated string or a list.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
+
+    gnupghome
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' gpg.receive_key keys='3FAD9F1E'
+        salt '*' gpg.receive_keys keys='3FAD9F1E'
 
-        salt '*' gpg.receive_key keys="['3FAD9F1E','3FBD9F2E']"
+        salt '*' gpg.receive_keys keys="['3FAD9F1E','3FBD9F2E']"
+
+        salt '*' gpg.receive_keys keys=3FAD9F1E user=username
 
-        salt '*' gpg.receive_key keys=3FAD9F1E user=username
     '''
     ret = {
            'res': True,
@@ -789,8 +870,9 @@ def trust_key(keyid=None,
         expired, unknown, not_trusted, marginally, fully, ultimately
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     CLI Example:
 
@@ -799,6 +881,7 @@ def trust_key(keyid=None,
         salt '*' gpg.trust_key keyid='3FAD9F1E' trust_level='marginally'
         salt '*' gpg.trust_key fingerprint='53C96788253E58416D20BCD352952C84C3252192' trust_level='not_trusted'
         salt '*' gpg.trust_key keys=3FAD9F1E trust_level='ultimately' user='username'
+
     '''
     ret = {
            'res': True,
@@ -816,7 +899,7 @@ def trust_key(keyid=None,
 
     if not fingerprint:
         if keyid:
-            key = get_key(keyid)
+            key = get_key(keyid, user=user)
             if key:
                 if 'fingerprint' not in key:
                     ret['res'] = False
@@ -835,15 +918,18 @@ def trust_key(keyid=None,
     if trust_level not in _VALID_TRUST_LEVELS:
         return 'ERROR: Valid trust levels - {0}'.format(','.join(_VALID_TRUST_LEVELS))
 
-    cmd = 'echo {0}:{1} | {2} --import-ownertrust'.format(_cmd_quote(fingerprint),
-                                                          _cmd_quote(NUM_TRUST_DICT[trust_level]),
-                                                          _cmd_quote(_check_gpg()))
+    stdin = '{0}:{1}\n'.format(fingerprint, NUM_TRUST_DICT[trust_level])
+    cmd = [_gpg(), '--import-ownertrust']
     _user = user
+
     if user == 'salt':
         homeDir = os.path.join(salt.syspaths.CONFIG_DIR, 'gpgkeys')
-        cmd = '{0} --homedir {1}'.format(cmd, homeDir)
+        cmd.extend([' --homedir', homeDir])
         _user = 'root'
-    res = __salt__['cmd.run_all'](cmd, runas=_user, python_shell=True)
+    res = __salt__['cmd.run_all'](cmd,
+                                  stdin=stdin,
+                                  runas=_user,
+                                  python_shell=False)
 
     if not res['retcode'] == 0:
         ret['res'] = False
@@ -876,8 +962,9 @@ def sign(user=None,
     Sign message or file
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     keyid
         The keyid of the key to set the trust level for, defaults to
@@ -893,10 +980,11 @@ def sign(user=None,
         The filename where the signed file will be written, default is standard out.
 
     use_passphrase
-        Whether to use a passphrase with the signing key.  Passphrase is received from pillar.
+        Whether to use a passphrase with the signing key. Passphrase is received
+        from Pillar.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -906,12 +994,12 @@ def sign(user=None,
 
         salt '*' gpg.sign filename='/path/to/important.file'
 
-        salt '*' gpg.sign filename='/path/to/important.file' use_pasphrase=True
+        salt '*' gpg.sign filename='/path/to/important.file' use_passphrase=True
 
     '''
     gpg = _create_gpg(user, gnupghome)
     if use_passphrase:
-        gpg_passphrase = __salt__['pillar.item']('gpg_passphrase')
+        gpg_passphrase = __salt__['pillar.get']('gpg_passphrase')
         if not gpg_passphrase:
             raise SaltInvocationError('gpg_passphrase not available in pillar.')
     else:
@@ -919,7 +1007,7 @@ def sign(user=None,
 
     # Check for at least one secret key to sign with
 
-    gnupg_version = distutils.version.LooseVersion(gnupg.__version__)
+    gnupg_version = _LooseVersion(gnupg.__version__)
     if text:
         if gnupg_version >= '1.3.1':
             signed_data = gpg.sign(text, default_key=keyid, passphrase=gpg_passphrase)
@@ -953,11 +1041,12 @@ def verify(text=None,
         The filename to verify.
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     CLI Example:
 
@@ -967,7 +1056,7 @@ def verify(text=None,
 
         salt '*' gpg.verify filename='/path/to/important.file'
 
-        salt '*' gpg.verify filename='/path/to/important.file' use_pasphrase=True
+        salt '*' gpg.verify filename='/path/to/important.file' use_passphrase=True
 
     '''
     gpg = _create_gpg(user)
@@ -1006,8 +1095,9 @@ def encrypt(user=None,
     Encrypt a message or file
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     recipients
         The fingerprints for those recipient whom the data is being encrypted for.
@@ -1022,18 +1112,19 @@ def encrypt(user=None,
         The filename where the signed file will be written, default is standard out.
 
     sign
-        Whether to sign, in addition to encrypt, the data.  True to use default key or fingerprint
-        to specify a different key to sign with.
+        Whether to sign, in addition to encrypt, the data. ``True`` to use
+        default key or fingerprint to specify a different key to sign with.
 
     use_passphrase
-        Whether to use a passphrase with the signing key.  Passphrase is received from pillar.
+        Whether to use a passphrase with the signing key. Passphrase is received
+        from Pillar.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     bare
-        If True, return the (armored) encrypted block as a string without the
-        standard comment/res dict
+        If ``True``, return the (armored) encrypted block as a string without
+        the standard comment/res dict.
 
     CLI Example:
 
@@ -1043,7 +1134,7 @@ def encrypt(user=None,
 
         salt '*' gpg.encrypt filename='/path/to/important.file'
 
-        salt '*' gpg.encrypt filename='/path/to/important.file' use_pasphrase=True
+        salt '*' gpg.encrypt filename='/path/to/important.file' use_passphrase=True
 
     '''
     ret = {
@@ -1053,7 +1144,7 @@ def encrypt(user=None,
     gpg = _create_gpg(user, gnupghome)
 
     if use_passphrase:
-        gpg_passphrase = __salt__['pillar.item']('gpg_passphrase')
+        gpg_passphrase = __salt__['pillar.get']('gpg_passphrase')
         if not gpg_passphrase:
             raise SaltInvocationError('gpg_passphrase not available in pillar.')
         gpg_passphrase = gpg_passphrase['gpg_passphrase']
@@ -1108,8 +1199,9 @@ def decrypt(user=None,
     Decrypt a message or file
 
     user
-        Which user's keychain to access, defaults to user Salt is running as.  Passing
-        the user as 'salt' will set the GPG home directory to /etc/salt/gpgkeys.
+        Which user's keychain to access, defaults to user Salt is running as.
+        Passing the user as ``salt`` will set the GnuPG home directory to the
+        ``/etc/salt/gpgkeys``.
 
     text
         The encrypted text to decrypt.
@@ -1121,14 +1213,15 @@ def decrypt(user=None,
         The filename where the decrypted data will be written, default is standard out.
 
     use_passphrase
-        Whether to use a passphrase with the signing key.  Passphrase is received from pillar.
+        Whether to use a passphrase with the signing key. Passphrase is received
+        from Pillar.
 
     gnupghome
-        Specify the location where GPG related files are stored.
+        Specify the location where GPG keyring and related files are stored.
 
     bare
-        If True, return the (armored) decrypted block as a string without the
-        standard comment/res dict
+        If ``True``, return the (armored) decrypted block as a string without the
+        standard comment/res dict.
 
     CLI Example:
 
@@ -1136,8 +1229,7 @@ def decrypt(user=None,
 
         salt '*' gpg.decrypt filename='/path/to/important.file.gpg'
 
-        salt '*' gpg.decrypt filename='/path/to/important.file.gpg' use_pasphrase=True
-
+        salt '*' gpg.decrypt filename='/path/to/important.file.gpg' use_passphrase=True
 
     '''
     ret = {
@@ -1146,7 +1238,7 @@ def decrypt(user=None,
     }
     gpg = _create_gpg(user, gnupghome)
     if use_passphrase:
-        gpg_passphrase = __salt__['pillar.item']('gpg_passphrase')
+        gpg_passphrase = __salt__['pillar.get']('gpg_passphrase')
         if not gpg_passphrase:
             raise SaltInvocationError('gpg_passphrase not available in pillar.')
         gpg_passphrase = gpg_passphrase['gpg_passphrase']
