@@ -1,6 +1,30 @@
 # -*- coding: utf-8 -*-
 '''
 Support for iptables
+
+Configuration Options
+---------------------
+
+The following options can be set in the :ref:`minion config
+<configuration-salt-minion>`, :ref:`minion grains<configuration-minion-grains>`
+, :ref:`minion pillar<configuration-minion-pillar>`, or
+`master config<configuration-salt-master>`.
+
+- ``iptables.save_filters``: List of REGEX strings to FILTER OUT matching lines
+
+    This is useful for filtering out chains, rules, etc that you do not
+    wish to persist, such as ephemeral Docker rules.
+
+    The default is to not filter out anything.
+
+    .. code-block:: yaml
+
+        iptables.save_filters:
+           - "-j CATTLE_PREROUTING"
+           - "-j DOCKER"
+           - "-A POSTROUTING"
+           - "-A CATTLE_POSTROUTING"
+           - "-A FORWARD"
 '''
 from __future__ import absolute_import
 
@@ -9,7 +33,6 @@ import os
 import re
 import sys
 import uuid
-import shlex
 import string
 
 # Import salt libs
@@ -27,7 +50,7 @@ def __virtual__():
     Only load the module if iptables is installed
     '''
     if not salt.utils.which('iptables'):
-        return False
+        return (False, 'The iptables execution module cannot be loaded: iptables not installed.')
 
     return True
 
@@ -84,12 +107,67 @@ def _conf(family='ipv4'):
     elif __grains__['os_family'] == 'Suse':
         # SuSE does not seem to use separate files for IPv4 and IPv6
         return '/etc/sysconfig/scripts/SuSEfirewall2-custom'
+    elif __grains__['os'] == 'Alpine':
+        if family == 'ipv6':
+            return '/etc/iptables/rules6-save'
+        else:
+            return '/etc/iptables/rules-save'
     else:
         raise SaltException('Saving iptables to file is not' +
                             ' supported on {0}.'.format(__grains__['os']) +
                             ' Please file an issue with SaltStack')
 
 
+def _conf_save_filters():
+    '''
+    Return array of strings from `save_filters` in config.
+
+    This array will be pulled from minion config, minion grains,
+    minion pillar, or master config.  The default value returned is [].
+
+    .. code-block:: python
+
+        _conf_save_filters()
+    '''
+    config = __salt__['config.option']('iptables.save_filters', [])
+    return config
+
+
+def _regex_iptables_save(cmd_output, filters=None):
+    '''
+    Return string with `save_filter` regex entries removed.  For example:
+
+    If `filters` is not provided, it will be pulled from minion config,
+    minion grains, minion pillar, or master config. Default return value
+    if no filters found is the original cmd_output string.
+
+    .. code-block:: python
+
+        _regex_iptables_save(cmd_output, ['-A DOCKER*'])
+    '''
+    # grab RE compiled filters from context for performance
+    if 'iptables.save_filters' not in __context__:
+        __context__['iptables.save_filters'] = []
+        for pattern in filters or _conf_save_filters():
+            try:
+                __context__['iptables.save_filters']\
+                    .append(re.compile(pattern))
+            except re.error as e:
+                log.warning('Skipping regex rule: \'{0}\': {1}'
+                            .format(pattern, e))
+                continue
+
+    if len(__context__['iptables.save_filters']) > 0:
+        # line by line get rid of any regex matches
+        _filtered_cmd_output = \
+            [line for line in cmd_output.splitlines(True)
+             if not any(reg.search(line) for reg
+                        in __context__['iptables.save_filters'])]
+        return ''.join(_filtered_cmd_output)
+
+    return cmd_output
+
+
 def version(family='ipv4'):
     '''
     Return version from iptables --version
@@ -127,6 +205,14 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
     To pass in jump options that doesn't take arguments, pass in an empty
     string.
 
+    .. note::
+
+        Whereas iptables will accept ``-p``, ``--proto[c[o[l]]]`` as synonyms
+        of ``--protocol``, if ``--proto`` appears in an iptables command after
+        the appearance of ``-m policy``, it is interpreted as the ``--proto``
+        option of the policy extension (see the iptables-extensions(8) man
+        page).
+
     CLI Examples:
 
     .. code-block:: bash
@@ -157,7 +243,6 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         salt '*' iptables.build_rule filter INPUT command=I position=3 \\
             full=True match=state state=RELATED,ESTABLISHED jump=ACCEPT \\
             family=ipv6
-
     '''
     if 'target' in kwargs:
         kwargs['jump'] = kwargs.pop('target')
@@ -171,7 +256,7 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             del kwargs[ignore]
 
     rule = []
-    proto = False
+    protocol = False
     bang_not_pat = re.compile(r'(!|not)\s?')
 
     def maybe_add_negation(arg):
@@ -195,12 +280,15 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         rule.append('{0}-o {1}'.format(maybe_add_negation('of'), kwargs['of']))
         del kwargs['of']
 
-    for proto_arg in ('protocol', 'proto'):
-        if proto_arg in kwargs:
-            if not proto:
-                rule.append('{0}-p {1}'.format(maybe_add_negation(proto_arg), kwargs[proto_arg]))
-                proto = True
-            del kwargs[proto_arg]
+    if 'proto' in kwargs and kwargs.get('match') != 'policy':
+        kwargs['protocol'] = kwargs['proto']
+        del kwargs['proto']
+        # Handle the case 'proto' in kwargs and kwargs.get('match') == 'policy' below
+    if 'protocol' in kwargs:
+        if not protocol:
+            rule.append('{0}-p {1}'.format(maybe_add_negation('protocol'), kwargs['protocol']))
+            protocol = True
+        del kwargs['protocol']
 
     if 'match' in kwargs:
         match_value = kwargs['match']
@@ -208,8 +296,12 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
             match_value = match_value.split(',')
         for match in match_value:
             rule.append('-m {0}'.format(match))
-            if 'name' in kwargs and match.strip() in ('pknock', 'quota2', 'recent'):
-                rule.append('--name {0}'.format(kwargs['name']))
+            if 'name_' in kwargs and match.strip() in ('pknock', 'quota2', 'recent'):
+                rule.append('--name {0}'.format(kwargs['name_']))
+                del kwargs['name_']
+        if 'proto' in kwargs and kwargs.get('match') == 'policy':
+            rule.append('{0}--proto {1}'.format(maybe_add_negation('proto'), kwargs['proto']))
+            del kwargs['proto']
         del kwargs['match']
 
     if 'match-set' in kwargs:
@@ -243,8 +335,8 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
         if multiport_arg in kwargs:
             if '-m multiport' not in rule:
                 rule.append('-m multiport')
-                if not proto:
-                    return 'Error: proto must be specified'
+                if not protocol:
+                    return 'Error: protocol must be specified'
 
             mp_value = kwargs[multiport_arg]
             if isinstance(mp_value, list):
@@ -406,46 +498,22 @@ def build_rule(table='filter', chain=None, command=None, position='', full=None,
     for after_jump_argument in after_jump_arguments:
         if after_jump_argument in kwargs:
             value = kwargs[after_jump_argument]
-            if any(ws_char in str(value) for ws_char in string.whitespace):
+            if value in (None, ''):  # options without arguments
+                after_jump.append('--{0}'.format(after_jump_argument))
+            elif any(ws_char in str(value) for ws_char in string.whitespace):
                 after_jump.append('--{0} "{1}"'.format(after_jump_argument, value))
             else:
                 after_jump.append('--{0} {1}'.format(after_jump_argument, value))
             del kwargs[after_jump_argument]
 
-    if 'log' in kwargs:
-        after_jump.append('--log {0} '.format(kwargs['log']))
-        del kwargs['log']
-
-    if 'log-level' in kwargs:
-        after_jump.append('--log-level {0} '.format(kwargs['log-level']))
-        del kwargs['log-level']
-
-    if 'log-prefix' in kwargs:
-        after_jump.append('--log-prefix {0} '.format(kwargs['log-prefix']))
-        del kwargs['log-prefix']
-
-    if 'log-tcp-sequence' in kwargs:
-        after_jump.append('--log-tcp-sequence {0} '.format(kwargs['log-tcp-sequence']))
-        del kwargs['log-tcp-sequence']
-
-    if 'log-tcp-options' in kwargs:
-        after_jump.append('--log-tcp-options {0} '.format(kwargs['log-tcp-options']))
-        del kwargs['log-tcp-options']
-
-    if 'log-ip-options' in kwargs:
-        after_jump.append('--log-ip-options {0} '.format(kwargs['log-ip-options']))
-        del kwargs['log-ip-options']
-
-    if 'log-uid' in kwargs:
-        after_jump.append('--log-uid {0} '.format(kwargs['log-uid']))
-        del kwargs['log-uid']
-
-    for item in kwargs:
-        rule.append(maybe_add_negation(item))
-        if len(item) == 1:
-            rule.append('-{0} {1}'.format(item, kwargs[item]))
-        else:
-            rule.append('--{0} {1}'.format(item, kwargs[item]))
+    for key in kwargs:
+        negation = maybe_add_negation(key)
+        # don't use .items() since maybe_add_negation removes the prefix from
+        # the value in the kwargs, thus we need to fetch it after that has run
+        value = kwargs[key]
+        flag = '-' if len(key) == 1 else '--'
+        value = '' if value in (None, '') else ' {0}'.format(value)
+        rule.append('{0}{1}{2}{3}'.format(negation, flag, key, value))
 
     rule += after_jump
 
@@ -602,6 +670,11 @@ def save(filename=None, family='ipv4'):
         os.makedirs(parent_dir)
     cmd = '{0}-save'.format(_iptables_cmd(family))
     ipt = __salt__['cmd.run'](cmd)
+
+    # regex out the output if configured with filters
+    if len(_conf_save_filters()) > 0:
+        ipt = _regex_iptables_save(ipt)
+
     out = __salt__['file.write'](filename, ipt)
     return out
 
@@ -776,6 +849,9 @@ def append(table='filter', chain=None, rule=None, family='ipv4'):
         return 'Error: Rule needs to be specified'
 
     wait = '--wait' if _has_option('--wait', family) else ''
+    returnCheck = check(table, chain, rule, family)
+    if isinstance(returnCheck, bool) and returnCheck:
+        return False
     cmd = '{0} {1} -t {2} -A {3} {4}'.format(
             _iptables_cmd(family), wait, table, chain, rule)
     out = __salt__['cmd.run'](cmd)
@@ -819,13 +895,16 @@ def insert(table='filter', chain=None, position=None, rule=None, family='ipv4'):
         return 'Error: Rule needs to be specified'
 
     if position < 0:
-        rules = get_rules(family='ipv4')
+        rules = get_rules(family=family)
         size = len(rules[table][chain]['rules'])
         position = (size + position) + 1
         if position is 0:
             position = 1
 
     wait = '--wait' if _has_option('--wait', family) else ''
+    returnCheck = check(table, chain, rule, family)
+    if isinstance(returnCheck, bool) and returnCheck:
+        return False
     cmd = '{0} {1} -t {2} -I {3} {4} {5}'.format(
             _iptables_cmd(family), wait, table, chain, position, rule)
     out = __salt__['cmd.run'](cmd)
@@ -928,7 +1007,7 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
             ret[table][chain]['rules'] = []
             ret[table][chain]['rules_comment'] = {}
         elif line.startswith('-A'):
-            args = shlex.split(line)
+            args = salt.utils.shlex_split(line)
             index = 0
             while index + 1 < len(args):
                 swap = args[index] == '!' and args[index + 1].startswith('-')
@@ -967,9 +1046,9 @@ def _parse_conf(conf_file=None, in_mem=False, family='ipv4'):
 
 def _parser():
     '''
-    This function contains _all_ the options I could find in man 8 iptables,
-    listed in the first section that I found them in. They will not all be used
-    by all parts of the module; use them intelligently and appropriately.
+    This function attempts to list all the options documented in the
+    iptables(8) and iptables-extensions(8) man pages.  They will not all be
+    used by all parts of the module; use them intelligently and appropriately.
     '''
     add_arg = None
     if sys.version.startswith('2.6'):
@@ -1389,6 +1468,8 @@ def _parser():
     add_arg('--or-mark', dest='or-mark', action='append')
     add_arg('--xor-mark', dest='xor-mark', action='append')
     add_arg('--set-mark', dest='set-mark', action='append')
+    add_arg('--nfmask', dest='nfmask', action='append')
+    add_arg('--ctmask', dest='ctmask', action='append')
     ## CONNSECMARK
     add_arg('--save', dest='save', action='append')
     add_arg('--restore', dest='restore', action='append')
