@@ -16,14 +16,13 @@ from salt.ext.six.moves import map
 
 # Third party libs
 try:
-    import win32gui
-    import win32con
     HAS_WIN32 = True
 except ImportError:
     HAS_WIN32 = False
 
 # Import salt libs
 import salt.utils
+import salt.utils.win_functions
 
 # Settings
 log = logging.getLogger(__name__)
@@ -35,7 +34,7 @@ def __virtual__():
     '''
     if salt.utils.is_windows() and HAS_WIN32:
         return 'win_path'
-    return False
+    return (False, "Module win_path: module only works on Windows systems")
 
 
 def _normalize_dir(string):
@@ -47,7 +46,15 @@ def _normalize_dir(string):
 
 def rehash():
     '''
-    Send a WM_SETTINGCHANGE Broadcast to Windows to refresh the Environment variables
+    Send a WM_SETTINGCHANGE Broadcast to Windows to refresh the Environment
+    variables for new processes.
+
+    .. note::
+        This will only affect new processes that aren't launched by services. To
+        apply changes to the path to services, the host must be restarted. The
+        ``salt-minion``, if running as a service, will not see changes to the
+        environment until the system is restarted. See
+        `MSDN Documentation <https://support.microsoft.com/en-us/help/821761/changes-that-you-make-to-environment-variables-do-not-affect-services>`_
 
     CLI Example:
 
@@ -55,12 +62,7 @@ def rehash():
 
         salt '*' win_path.rehash
     '''
-    return win32gui.SendMessageTimeout(win32con.HWND_BROADCAST,
-                                       win32con.WM_SETTINGCHANGE,
-                                       0,
-                                       'Environment',
-                                       0,
-                                       10000)[0] == 1
+    return salt.utils.win_functions.broadcast_setting_change('Environment')
 
 
 def get_path():
@@ -73,9 +75,9 @@ def get_path():
 
         salt '*' win_path.get_path
     '''
-    ret = __salt__['reg.read_key']('HKEY_LOCAL_MACHINE',
+    ret = __salt__['reg.read_value']('HKEY_LOCAL_MACHINE',
                                    'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment',
-                                   'PATH').split(';')
+                                   'PATH')['vdata'].split(';')
 
     # Trim ending backslash
     return list(map(_normalize_dir, ret))
@@ -148,7 +150,7 @@ def add(path, index=0):
 
     # Add it to the Path
     sysPath.insert(index, path)
-    regedit = __salt__['reg.set_key'](
+    regedit = __salt__['reg.set_value'](
         'HKEY_LOCAL_MACHINE',
         'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment',
         'PATH',
@@ -190,7 +192,7 @@ def remove(path):
     except ValueError:
         return True
 
-    regedit = __salt__['reg.set_key'](
+    regedit = __salt__['reg.set_value'](
         'HKEY_LOCAL_MACHINE',
         'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment',
         'PATH',
