@@ -8,13 +8,17 @@ from __future__ import absolute_import, print_function
 import os
 import sys
 import time
+import signal
 import logging
+import functools
 import threading
 import traceback
+import signal
+import functools
 from random import randint
 
 # Import salt libs
-from salt.exceptions import SaltSystemExit, SaltClientError, SaltReqTimeoutError, get_error_message
+from salt.exceptions import SaltSystemExit, SaltClientError, SaltReqTimeoutError
 import salt.defaults.exitcodes  # pylint: disable=unused-import
 
 log = logging.getLogger(__name__)
@@ -38,6 +42,52 @@ def _handle_interrupt(exc, original_exc, hardfail=False, trace=''):
         raise exc
 
 
+def _handle_signals(client, signum, sigframe):
+    try:
+        # This raises AttributeError on Python 3.4 and 3.5 if there is no current exception.
+        # Ref: https://bugs.python.org/issue23003
+        trace = traceback.format_exc()
+    except AttributeError:
+        trace = ''
+    try:
+        hardcrash = client.options.hard_crash
+    except (AttributeError, KeyError):
+        hardcrash = False
+
+    if signum == signal.SIGINT:
+        exit_msg = '\nExiting gracefully on Ctrl-c'
+        try:
+            jid = client.local_client.pub_data['jid']
+            exit_msg += (
+                '\n'
+                'This job\'s jid is: {0}\n'
+                'The minions may not have all finished running and any remaining '
+                'minions will return upon completion. To look up the return data '
+                'for this job later, run the following command:\n\n'
+                'salt-run jobs.lookup_jid {0}'.format(jid)
+            )
+        except (AttributeError, KeyError):
+            pass
+    else:
+        exit_msg = None
+
+    _handle_interrupt(
+        SystemExit(exit_msg),
+        Exception('\nExiting with hard crash on Ctrl-c'),
+        hardcrash, trace=trace)
+
+
+def _install_signal_handlers(client):
+    # Install the SIGINT/SIGTERM handlers if not done so far
+    if signal.getsignal(signal.SIGINT) is signal.SIG_DFL:
+        # No custom signal handling was added, install our own
+        signal.signal(signal.SIGINT, functools.partial(_handle_signals, client))
+
+    if signal.getsignal(signal.SIGTERM) is signal.SIG_DFL:
+        # No custom signal handling was added, install our own
+        signal.signal(signal.SIGINT, functools.partial(_handle_signals, client))
+
+
 def salt_master():
     '''
     Start the salt master.
@@ -47,19 +97,26 @@ def salt_master():
     master.start()
 
 
-def minion_process(queue):
+def minion_process():
     '''
     Start a minion process
     '''
+    import salt.utils
     import salt.cli.daemons
+
     # salt_minion spawns this function in a new process
 
+    salt.utils.appendproctitle('KeepAlive')
+
+    def handle_hup(manager, sig, frame):
+        manager.minion.reload()
+
     def suicide_when_without_parent(parent_pid):
         '''
         Have the minion suicide if the parent process is gone
 
-        NOTE: there is a small race issue where the parent PID could be replace
-        with another process with the same PID!
+        NOTE: small race issue where the parent PID could be replace
+        with another process with same PID!
         '''
         while True:
             time.sleep(5)
@@ -71,50 +128,38 @@ def minion_process(queue):
                 # forcibly exit, regular sys.exit raises an exception-- which
                 # isn't sufficient in a thread
                 log.error('Minion process encountered exception: {0}'.format(exc))
-                os._exit(999)
+                os._exit(salt.defaults.exitcodes.EX_GENERIC)
+
     if not salt.utils.is_windows():
         thread = threading.Thread(target=suicide_when_without_parent, args=(os.getppid(),))
         thread.start()
 
-    restart = False
-    minion = None
-    try:
-        minion = salt.cli.daemons.Minion()
-        minion.start()
-    except ImportError as exc:
-        log.error(exc)
-    except (Exception, SaltClientError, SaltReqTimeoutError, SaltSystemExit) as exc:
-        log.error(
-            'Minion failed to start: {0}'.format(get_error_message(exc)),
-            exc_info=True
-        )
-        restart = True
-    except SystemExit as exc:
-        log.error(exc)
+    minion = salt.cli.daemons.Minion()
+    signal.signal(signal.SIGHUP,
+                  functools.partial(handle_hup,
+                                    minion))
 
-    if restart is True:
+    try:
+        minion.start()
+    except (SaltClientError, SaltReqTimeoutError, SaltSystemExit) as exc:
         log.warning('Fatal functionality error caught by minion handler:\n', exc_info=True)
-        log.warn('** Restarting minion **')
+        log.warning('** Restarting minion **')
         delay = 60
-        if minion is not None:
-            if hasattr(minion, 'config'):
-                delay = minion.config.get('random_reauth_delay', 60)
-        random_delay = randint(1, delay)
-        log.info('Sleeping random_reauth_delay of {0} seconds'.format(random_delay))
-        # preform delay after minion resources have been cleaned
-        if minion.options.daemon:
-            time.sleep(random_delay)
-            salt_minion()
-        else:
-            queue.put(random_delay)
-    else:
-        queue.put(0)
+        if minion is not None and hasattr(minion, 'config'):
+            delay = minion.config.get('random_reauth_delay', 60)
+        delay = randint(1, delay)
+        log.info('waiting random_reauth_delay {0}s'.format(delay))
+        time.sleep(delay)
+        sys.exit(salt.defaults.exitcodes.SALT_KEEPALIVE)
 
 
 def salt_minion():
     '''
-    Start the salt minion.
+    Start the salt minion in a subprocess.
+    Auto restart minion on error.
     '''
+    import signal
+
     import salt.utils.process
     salt.utils.process.notify_systemd()
 
@@ -123,39 +168,60 @@ def salt_minion():
     if '' in sys.path:
         sys.path.remove('')
 
+    if salt.utils.is_windows():
+        minion = salt.cli.daemons.Minion()
+        minion.start()
+        return
+
     if '--disable-keepalive' in sys.argv:
         sys.argv.remove('--disable-keepalive')
         minion = salt.cli.daemons.Minion()
         minion.start()
         return
 
+    def escalate_signal_to_process(pid, signum, sigframe):  # pylint: disable=unused-argument
+        '''
+        Escalate the signal received to the multiprocessing process that
+        is actually running the minion
+        '''
+        # escalate signal
+        os.kill(pid, signum)
+
     # keep one minion subprocess running
+    prev_sigint_handler = signal.getsignal(signal.SIGINT)
+    prev_sigterm_handler = signal.getsignal(signal.SIGTERM)
     while True:
         try:
-            queue = multiprocessing.Queue()
-        except Exception:
-            # This breaks in containers
+            process = multiprocessing.Process(target=minion_process)
+            process.start()
+            signal.signal(signal.SIGTERM,
+                          functools.partial(escalate_signal_to_process,
+                                            process.pid))
+            signal.signal(signal.SIGINT,
+                          functools.partial(escalate_signal_to_process,
+                                            process.pid))
+            signal.signal(signal.SIGHUP,
+                          functools.partial(escalate_signal_to_process,
+                                            process.pid))
+        except Exception:  # pylint: disable=broad-except
+            # if multiprocessing does not work
             minion = salt.cli.daemons.Minion()
             minion.start()
-            return
-        process = multiprocessing.Process(target=minion_process, args=(queue,))
-        process.start()
-        try:
-            process.join()
-            try:
-                restart_delay = queue.get(block=False)
-            except Exception:
-                if process.exitcode == 0:
-                    # Minion process ended naturally, Ctrl+C or --version
-                    break
-                restart_delay = 60
-            if restart_delay == 0:
-                # Minion process ended naturally, Ctrl+C, --version, etc.
-                break
-            # delay restart to reduce flooding and allow network resources to close
-            time.sleep(restart_delay)
-        except KeyboardInterrupt:
             break
+
+        process.join()
+
+        # Process exited or was terminated. Since we're going to try to restart
+        # it, we MUST, reset signal handling to the previous handlers
+        signal.signal(signal.SIGINT, prev_sigint_handler)
+        signal.signal(signal.SIGTERM, prev_sigterm_handler)
+
+        if not process.exitcode == salt.defaults.exitcodes.SALT_KEEPALIVE:
+            sys.exit(process.exitcode)
+        # ontop of the random_reauth_delay already preformed
+        # delay extra to reduce flooding and free resources
+        # NOTE: values are static but should be fine.
+        time.sleep(2 + randint(1, 10))
         # need to reset logging because new minion objects
         # cause extra log handlers to accumulate
         rlogger = logging.getLogger()
@@ -187,23 +253,28 @@ def proxy_minion_process(queue):
                 # forcibly exit, regular sys.exit raises an exception-- which
                 # isn't sufficient in a thread
                 os._exit(999)
+
     if not salt.utils.is_windows():
         thread = threading.Thread(target=suicide_when_without_parent, args=(os.getppid(),))
         thread.start()
 
     restart = False
     proxyminion = None
+    status = salt.defaults.exitcodes.EX_OK
     try:
         proxyminion = salt.cli.daemons.ProxyMinion()
         proxyminion.start()
     except (Exception, SaltClientError, SaltReqTimeoutError, SaltSystemExit) as exc:
         log.error('Proxy Minion failed to start: ', exc_info=True)
         restart = True
+        # status is superfluous since the process will be restarted
+        status = salt.defaults.exitcodes.SALT_KEEPALIVE
     except SystemExit as exc:
         restart = False
+        status = exc.code
 
     if restart is True:
-        log.warn('** Restarting proxy minion **')
+        log.warning('** Restarting proxy minion **')
         delay = 60
         if proxyminion is not None:
             if hasattr(proxyminion, 'config'):
@@ -214,9 +285,10 @@ def proxy_minion_process(queue):
         queue.put(random_delay)
     else:
         queue.put(0)
+    sys.exit(status)
 
 
-def salt_proxy_minion():
+def salt_proxy():
     '''
     Start a proxy minion.
     '''
@@ -258,7 +330,7 @@ def salt_proxy_minion():
                 restart_delay = 60
             if restart_delay == 0:
                 # Minion process ended naturally, Ctrl+C, --version, etc.
-                break
+                sys.exit(process.exitcode)
             # delay restart to reduce flooding and allow network resources to close
             time.sleep(restart_delay)
         except KeyboardInterrupt:
@@ -292,22 +364,12 @@ def salt_key():
     Manage the authentication keys with salt-key.
     '''
     import salt.cli.key
-    client = None
     try:
         client = salt.cli.key.SaltKey()
+        _install_signal_handlers(client)
         client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
     except Exception as err:
-        sys.stderr.write("Error: {0}\n".format(err.message))
+        sys.stderr.write("Error: {0}\n".format(err))
 
 
 def salt_cp():
@@ -316,20 +378,9 @@ def salt_cp():
     master.
     '''
     import salt.cli.cp
-    client = None
-    try:
-        client = salt.cli.cp.SaltCPCli()
-        client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
+    client = salt.cli.cp.SaltCPCli()
+    _install_signal_handlers(client)
+    client.run()
 
 
 def salt_call():
@@ -340,20 +391,9 @@ def salt_call():
     import salt.cli.call
     if '' in sys.path:
         sys.path.remove('')
-    client = None
-    try:
-        client = salt.cli.call.SaltCall()
-        client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
+    client = salt.cli.call.SaltCall()
+    _install_signal_handlers(client)
+    client.run()
 
 
 def salt_run():
@@ -363,20 +403,9 @@ def salt_run():
     import salt.cli.run
     if '' in sys.path:
         sys.path.remove('')
-    client = None
-    try:
-        client = salt.cli.run.SaltRun()
-        client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
+    client = salt.cli.run.SaltRun()
+    _install_signal_handlers(client)
+    client.run()
 
 
 def salt_ssh():
@@ -386,20 +415,10 @@ def salt_ssh():
     import salt.cli.ssh
     if '' in sys.path:
         sys.path.remove('')
-    client = None
     try:
         client = salt.cli.ssh.SaltSSH()
+        _install_signal_handlers(client)
         client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
     except SaltClientError as err:
         trace = traceback.format_exc()
         try:
@@ -416,34 +435,25 @@ def salt_cloud():
     '''
     The main function for salt-cloud
     '''
+    # Define 'salt' global so we may use it after ImportError. Otherwise,
+    # UnboundLocalError will be raised.
+    global salt  # pylint: disable=W0602
+
     try:
+        # Late-imports for CLI performance
+        import salt.cloud
         import salt.cloud.cli
-        has_saltcloud = True
     except ImportError as e:
-        log.error("Error importing salt cloud {0}".format(e))
         # No salt cloud on Windows
-        has_saltcloud = False
+        log.error("Error importing salt cloud {0}".format(e))
+        print('salt-cloud is not available in this system')
+        sys.exit(salt.defaults.exitcodes.EX_UNAVAILABLE)
     if '' in sys.path:
         sys.path.remove('')
 
-    if not has_saltcloud:
-        print('salt-cloud is not available in this system')
-        sys.exit(salt.defaults.exitcodes.EX_UNAVAILABLE)
-
-    client = None
-    try:
-        client = salt.cloud.cli.SaltCloud()
-        client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
+    client = salt.cloud.cli.SaltCloud()
+    _install_signal_handlers(client)
+    client.run()
 
 
 def salt_api():
@@ -455,7 +465,7 @@ def salt_api():
 
     import salt.cli.api
     sapi = salt.cli.api.SaltAPI()  # pylint: disable=E1120
-    sapi.run()
+    sapi.start()
 
 
 def salt_main():
@@ -466,20 +476,9 @@ def salt_main():
     import salt.cli.salt
     if '' in sys.path:
         sys.path.remove('')
-    client = None
-    try:
-        client = salt.cli.salt.SaltCMD()
-        client.run()
-    except KeyboardInterrupt as err:
-        trace = traceback.format_exc()
-        try:
-            hardcrash = client.options.hard_crash
-        except (AttributeError, KeyError):
-            hardcrash = False
-        _handle_interrupt(
-            SystemExit('\nExiting gracefully on Ctrl-c'),
-            err,
-            hardcrash, trace=trace)
+    client = salt.cli.salt.SaltCMD()
+    _install_signal_handlers(client)
+    client.run()
 
 
 def salt_spm():
@@ -491,3 +490,17 @@ def salt_spm():
     import salt.cli.spm
     spm = salt.cli.spm.SPM()  # pylint: disable=E1120
     spm.run()
+
+
+def salt_extend(extension, name, description, salt_dir, merge):
+    '''
+    Quickstart for developing on the saltstack installation
+
+    .. versionadded:: 2016.11.0
+    '''
+    import salt.utils.extend
+    salt.utils.extend.run(extension=extension,
+                          name=name,
+                          description=description,
+                          salt_dir=salt_dir,
+                          merge=merge)
