@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 '''
-Management of APT/YUM package repos
-===================================
+Management of APT/DNF/YUM/Zypper package repos
+==============================================
 
-Package repositories for APT-based and YUM-based distros can be managed with
-these states. Here is some example SLS:
+States for managing software package repositories on Linux distros. Supported
+package managers are APT, DNF, YUM and Zypper. Here is some example SLS:
 
 .. code-block:: yaml
 
@@ -92,10 +92,10 @@ import sys
 
 # Import salt libs
 from salt.exceptions import CommandExecutionError, SaltInvocationError
-from salt.modules.aptpkg import _strip_uri
 from salt.state import STATE_INTERNAL_KEYWORDS as _STATE_INTERNAL_KEYWORDS
 import salt.utils
 import salt.utils.pkg.deb
+import salt.utils.pkg.rpm
 
 
 def __virtual__():
@@ -131,7 +131,7 @@ def managed(name, ppa=None, **kwargs):
 
     disabled : False
         Included to reduce confusion due to APT's use of the ``disabled``
-        argument. If this is passed for a yum/dnf/zypper-based distro, then the
+        argument. If this is passed for a YUM/DNF/Zypper-based distro, then the
         reverse will be passed as ``enabled``. For example passing
         ``disabled=True`` will assume ``enabled=False``.
 
@@ -150,8 +150,14 @@ def managed(name, ppa=None, **kwargs):
         enabled configuration. Anything supplied for this list will be saved
         in the repo configuration with a comment marker (#) in front.
 
-    Additional configuration values seen in yum repo files, such as ``gpgkey`` or
-    ``gpgcheck``, will be used directly as key-value pairs. For example:
+    gpgautoimport
+        Only valid for Zypper package manager. If set to True, automatically
+        trust and import public GPG key for the repository. The key should be
+        specified with ``gpgkey`` parameter. See details below.
+
+    Additional configuration values seen in YUM/DNF/Zypper repo files, such as
+    ``gpgkey`` or ``gpgcheck``, will be used directly as key-value pairs.
+    For example:
 
     .. code-block:: yaml
 
@@ -217,7 +223,7 @@ def managed(name, ppa=None, **kwargs):
         and/or installing packages.
 
     enabled : True
-        Included to reduce confusion due to yum/dnf/zypper's use of the
+        Included to reduce confusion due to YUM/DNF/Zypper's use of the
         ``enabled`` argument. If this is passed for an APT-based distro, then
         the reverse will be passed as ``disabled``. For example, passing
         ``enabled=False`` will assume ``disabled=False``.
@@ -368,9 +374,12 @@ def managed(name, ppa=None, **kwargs):
         sanitizedkwargs = kwargs
 
     if os_family == 'debian':
-        repo = _strip_uri(repo)
+        repo = salt.utils.pkg.deb.strip_uri(repo)
 
     if pre:
+        #22412: Remove file attribute in case same repo is set up multiple times but with different files
+        pre.pop('file', None)
+        sanitizedkwargs.pop('file', None)
         for kwarg in sanitizedkwargs:
             if kwarg not in pre:
                 if kwarg == 'enabled':
@@ -403,6 +412,12 @@ def managed(name, ppa=None, **kwargs):
                         salt.utils.pkg.deb.combine_comments(kwargs['comments'])
                     if pre_comments != post_comments:
                         break
+            elif kwarg == 'comments' and os_family == 'redhat':
+                precomments = salt.utils.pkg.rpm.combine_comments(pre[kwarg])
+                kwargcomments = salt.utils.pkg.rpm.combine_comments(
+                        sanitizedkwargs[kwarg])
+                if precomments != kwargcomments:
+                    break
             else:
                 if os_family in ('redhat', 'suse') \
                         and any(isinstance(x, bool) for x in
