@@ -334,6 +334,7 @@ import errno
 import random
 import yaml
 import copy
+import weakref
 
 # Import Salt libs
 import salt.config
@@ -381,7 +382,7 @@ class Schedule(object):
     '''
     instance = None
 
-    def __new__(cls, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None):
+    def __new__(cls, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None, utils=None):
         '''
         Only create one instance of Schedule
         '''
@@ -391,20 +392,21 @@ class Schedule(object):
             # it in a WeakValueDictionary-- which will remove the item if no one
             # references it-- this forces a reference while we return to the caller
             cls.instance = object.__new__(cls)
-            cls.instance.__singleton_init__(opts, functions, returners, intervals, cleanup, proxy)
+            cls.instance.__singleton_init__(opts, functions, returners, intervals, cleanup, proxy, utils)
         else:
             log.debug('Re-using Schedule')
         return cls.instance
 
     # has to remain empty for singletons, since __init__ will *always* be called
-    def __init__(self, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None):
+    def __init__(self, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None, utils=None):
         pass
 
     # an init for the singleton instance to call
-    def __singleton_init__(self, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None):
+    def __singleton_init__(self, opts, functions, returners=None, intervals=None, cleanup=None, proxy=None, utils=None):
         self.opts = opts
         self.proxy = proxy
         self.functions = functions
+        self.utils = utils
         if isinstance(intervals, dict):
             self.intervals = intervals
         else:
@@ -466,9 +468,6 @@ class Schedule(object):
             config_dir,
             os.path.dirname(self.opts.get('default_include',
                                           salt.config.DEFAULT_MINION_OPTS['default_include'])))
-        if salt.utils.is_proxy():
-            # each proxy will have a separate _schedule.conf file
-            minion_d_dir = os.path.join(minion_d_dir, self.opts['proxyid'])
 
         if not os.path.isdir(minion_d_dir):
             os.makedirs(minion_d_dir)
@@ -520,7 +519,7 @@ class Schedule(object):
         for job in list(self.opts['schedule'].keys()):
             if job.startswith(name):
                 del self.opts['schedule'][job]
-        for job in self._get_schedule(include_opts=False).keys():
+        for job in self._get_schedule(include_opts=False):
             if job.startswith(name):
                 log.warning('Cannot delete job {0}, '
                             'it`s in the pillar!'.format(job))
@@ -554,7 +553,7 @@ class Schedule(object):
 
         # if enabled is not included in the job,
         # assume job is enabled.
-        for job in data.keys():
+        for job in data:
             if 'enabled' not in data[job]:
                 data[job]['enabled'] = True
 
@@ -753,10 +752,11 @@ class Schedule(object):
             # This also needed for ZeroMQ transport to reset all functions
             # context data that could keep paretns connections. ZeroMQ will
             # hang on polling parents connections from the child process.
+            utils = self.utils or salt.loader.utils(self.opts)
             if self.opts['__role'] == 'master':
-                self.functions = salt.loader.runner(self.opts)
+                self.functions = salt.loader.runner(self.opts, utils=utils)
             else:
-                self.functions = salt.loader.minion_mods(self.opts, proxy=self.proxy)
+                self.functions = salt.loader.minion_mods(self.opts, proxy=self.proxy, utils=utils)
             self.returners = salt.loader.returners(self.opts, self.functions, proxy=self.proxy)
         ret = {'id': self.opts.get('id', 'master'),
                'fun': func,
@@ -848,6 +848,48 @@ class Schedule(object):
                     if key is not 'kwargs':
                         kwargs['__pub_{0}'.format(key)] = copy.deepcopy(val)
 
+            # Only include these when running runner modules
+            if self.opts['__role'] == 'master':
+                jid = salt.utils.jid.gen_jid()
+                tag = salt.utils.event.tagify(jid, prefix='salt/scheduler/')
+
+                event = salt.utils.event.get_event(
+                        self.opts['__role'],
+                        self.opts['sock_dir'],
+                        self.opts['transport'],
+                        opts=self.opts,
+                        listen=False)
+
+                namespaced_event = salt.utils.event.NamespacedEvent(
+                    event,
+                    tag,
+                    print_func=None
+                )
+
+                func_globals = {
+                    '__jid__': jid,
+                    '__user__': salt.utils.get_user(),
+                    '__tag__': tag,
+                    '__jid_event__': weakref.proxy(namespaced_event),
+                }
+                self_functions = copy.copy(self.functions)
+                salt.utils.lazy.verify_fun(self_functions, func)
+
+                # Inject some useful globals to *all* the function's global
+                # namespace only once per module-- not per func
+                completed_funcs = []
+
+                for mod_name in six.iterkeys(self_functions):
+                    if '.' not in mod_name:
+                        continue
+                    mod, _ = mod_name.split('.', 1)
+                    if mod in completed_funcs:
+                        continue
+                    completed_funcs.append(mod)
+                    for global_key, value in six.iteritems(func_globals):
+                        self.functions[mod_name].__globals__[global_key] = value
+
+            self.functions.pack['__context__']['retcode'] = 0
             ret['return'] = self.functions[func](*args, **kwargs)
 
             # runners do not provide retcode
@@ -943,7 +985,26 @@ class Schedule(object):
         '''
         Evaluate and execute the schedule
         '''
+
+        def _splay(splaytime):
+            '''
+            Calculate splaytime
+            '''
+            splay_ = None
+            if isinstance(splaytime, dict):
+                if splaytime['end'] >= splaytime['start']:
+                    splay_ = random.randint(splaytime['start'],
+                                            splaytime['end'])
+                else:
+                    log.error('schedule.handle_func: Invalid Splay, '
+                              'end must be larger than start. Ignoring splay.')
+            else:
+                splay_ = random.randint(1, splaytime)
+            return splay_
+
         schedule = self._get_schedule()
+        if not isinstance(schedule, dict):
+            raise ValueError('Schedule must be of type dict.')
         if 'enabled' in schedule and not schedule['enabled']:
             return
         for job, data in six.iteritems(schedule):
@@ -971,9 +1032,18 @@ class Schedule(object):
                 )
             if 'name' not in data:
                 data['name'] = job
-            # Add up how many seconds between now and then
-            when = 0
-            seconds = 0
+
+            if '_next_fire_time' not in data:
+                data['_next_fire_time'] = None
+
+            if '_splay' not in data:
+                data['_splay'] = None
+
+            if 'run_on_start' in data and \
+                    data['run_on_start'] and \
+                    '_run_on_start' not in data:
+                data['_run_on_start'] = True
+
             now = int(time.time())
 
             if 'until' in data:
@@ -1029,26 +1099,40 @@ class Schedule(object):
                 continue
 
             if True in [True for item in time_elements if item in data]:
-                # Add up how many seconds between now and then
-                seconds += int(data.get('seconds', 0))
-                seconds += int(data.get('minutes', 0)) * 60
-                seconds += int(data.get('hours', 0)) * 3600
-                seconds += int(data.get('days', 0)) * 86400
+                if '_seconds' not in data:
+                    interval = int(data.get('seconds', 0))
+                    interval += int(data.get('minutes', 0)) * 60
+                    interval += int(data.get('hours', 0)) * 3600
+                    interval += int(data.get('days', 0)) * 86400
+
+                    data['_seconds'] = interval
+
+                    if not data['_next_fire_time']:
+                        data['_next_fire_time'] = now + data['_seconds']
+
+                    if interval < self.loop_interval:
+                        self.loop_interval = interval
+
             elif 'once' in data:
-                once_fmt = data.get('once_fmt', '%Y-%m-%dT%H:%M:%S')
-
-                try:
-                    once = datetime.datetime.strptime(data['once'], once_fmt)
-                    once = int(time.mktime(once.timetuple()))
-                except (TypeError, ValueError):
-                    log.error('Date string could not be parsed: %s, %s',
-                            data['once'], once_fmt)
+                if data['_next_fire_time'] and \
+                        data['_next_fire_time'] != now and \
+                        not data['_splay']:
                     continue
 
-                if now != once:
-                    continue
-                else:
-                    seconds = 1
+                if not data['_next_fire_time'] and \
+                        not data['_splay']:
+                    once_fmt = data.get('once_fmt', '%Y-%m-%dT%H:%M:%S')
+                    try:
+                        once = datetime.datetime.strptime(data['once'],
+                                                          once_fmt)
+                        data['_next_fire_time'] = int(
+                            time.mktime(once.timetuple()))
+                    except (TypeError, ValueError):
+                        log.error('Date string could not be parsed: %s, %s',
+                                  data['once'], once_fmt)
+                        continue
+                    if data['_next_fire_time'] != now:
+                        continue
 
             elif 'when' in data:
                 if not _WHEN_SUPPORTED:
@@ -1094,10 +1178,14 @@ class Schedule(object):
                                 continue
                         _when.append(int(time.mktime(when__.timetuple())))
 
+                    if data['_splay']:
+                        _when.append(data['_splay'])
+
                     # Sort the list of "whens" from earlier to later schedules
                     _when.sort()
 
-                    for i in _when:
+                    # Copy the list so we can loop through it
+                    for i in copy.deepcopy(_when):
                         if i < now and len(_when) > 1:
                             # Remove all missed schedules except the latest one.
                             # We need it to detect if it was triggered previously.
@@ -1108,38 +1196,20 @@ class Schedule(object):
                         # last scheduled time in the past.
                         when = _when[0]
 
-                        # If we're switching to the next run in a list
-                        # ensure the job can run.
-                        if '_when' in data and \
-                                (data['_when'] != when or len(_when) == 1):
-                            data['_when_run'] = True
-                            # Calculate time to previously scheduled job
-                            # (which has not been run yet), or it should be
-                            # the last time definition in the list.
-                            seconds = data['_when'] - now
-                        else:
-                            seconds = when - now
+                        if '_run' not in data:
+                            # Prevent run of jobs from the past
+                            data['_run'] = bool(when >= now)
 
-                        # Scheduled time is in the past and the run was not triggered before
-                        if seconds < 0 and not data.get('_when_run', False):
-                            continue
+                        if not data['_next_fire_time']:
+                            data['_next_fire_time'] = when
 
-                        if '_when_run' not in data:
-                            data['_when_run'] = True
+                        if data['_next_fire_time'] < when and \
+                                not data['_run']:
+                            data['_next_fire_time'] = when
+                            data['_run'] = True
 
-                        # Backup the run time
-                        if '_when' not in data:
-                            data['_when'] = when
-
-                        # A new 'when' ensure _when_run is True
-                        if when > data['_when']:
-                            data['_when'] = when
-                            data['_when_run'] = True
-                        # At last scheduled time, we disable all subsequent job runs
-                        elif len(_when) == 1:
-                            del data['_when']
-
-                    else:
+                    elif not data.get('_run', False):
+                        data['_next_fire_time'] = None
                         continue
 
                 else:
@@ -1173,108 +1243,91 @@ class Schedule(object):
                             log.error('Invalid date string. Ignoring')
                             continue
                     when = int(time.mktime(when__.timetuple()))
-                    now = int(time.time())
-                    seconds = when - now
 
-                    # scheduled time is in the past and the run was not triggered before
-                    if seconds < 0 and not data.get('_when_run', False):
+                    if when < now and \
+                            not data.get('_run', False) and \
+                            not data['_splay']:
+                        data['_next_fire_time'] = None
                         continue
 
-                    if '_when_run' not in data:
-                        data['_when_run'] = True
+                    if '_run' not in data:
+                        data['_run'] = True
 
-                    # Backup the run time
-                    if '_when' not in data:
-                        data['_when'] = when
+                    if not data['_next_fire_time']:
+                        data['_next_fire_time'] = when
 
-                    # A new 'when' ensure _when_run is True
-                    if when > data['_when']:
-                        data['_when'] = when
-                        data['_when_run'] = True
+                    if data['_next_fire_time'] < when and \
+                            not data['_run']:
+                        data['_next_fire_time'] = when
+                        data['_run'] = True
 
             elif 'cron' in data:
                 if not _CRON_SUPPORTED:
                     log.error('Missing python-croniter. Ignoring job {0}'.format(job))
                     continue
 
-                now = int(time.mktime(datetime.datetime.now().timetuple()))
-                try:
-                    cron = croniter.croniter(data['cron'], now)
-                    # Get first scheduled time from now on to be able to
-                    # calculate when it was intended to run before.
-                    cron.get_next()
-                except (ValueError, KeyError):
-                    log.error('Invalid cron string. Ignoring')
-                    continue
-                # Calculate how many seconds passed from previous run...
-                data['_cron_prev'] = now - int(cron.get_prev())
-                # ...and need to wait till next scheduled job.
-                data['_cron_next'] = int(cron.get_next()) - now
-                # If less than one minute left till next job, this will
-                # increase loop interval to wait for next cycle.
-                # This prevents setting loop interval to low, which has
-                # negative performance impact, especially on Master side.
-                seconds = data['_cron_next'] if data['_cron_next'] >= 60 \
-                    else int(cron.get_next()) - now
+                if data['_next_fire_time'] is None:
+                    # Get next time frame for a "cron" job if it has been never
+                    # executed before or already executed in the past.
+                    try:
+                        data['_next_fire_time'] = int(
+                            croniter.croniter(data['cron'], now).get_next())
+                    except (ValueError, KeyError):
+                        log.error('Invalid cron string. Ignoring')
+                        continue
+
+                    # If next job run is scheduled more than 1 minute ahead and
+                    # configured loop interval is longer than that, we should
+                    # shorten it to get our job executed closer to the beginning
+                    # of desired time.
+                    interval = now - data['_next_fire_time']
+                    if interval >= 60 and interval < self.loop_interval:
+                        self.loop_interval = interval
+
             else:
                 continue
 
-            # Check if the seconds variable is lower than current lowest
-            # loop interval needed. If it is lower than overwrite variable
-            # external loops using can then check this variable for how often
-            # they need to reschedule themselves
-            # Not used with 'when' parameter, causes run away jobs and CPU
-            # spikes.
-            if 'when' not in data:
-                if seconds < self.loop_interval:
-                    self.loop_interval = seconds
-
             run = False
+            seconds = data['_next_fire_time'] - now
 
             if 'splay' in data:
-                if 'when' in data:
-                    log.error('Unable to use "splay" with "when" option at this time. Ignoring.')
-                elif 'cron' in data:
-                    log.error('Unable to use "splay" with "cron" option at this time. Ignoring.')
-                else:
-                    if '_seconds' not in data:
-                        log.debug('The _seconds parameter is missing, '
-                                  'most likely the first run or the schedule '
-                                  'has been refreshed refresh.')
-                        if 'seconds' in data:
-                            data['_seconds'] = data['seconds']
-                        else:
-                            data['_seconds'] = 0
-
-            if 'when' in data:
-                # scheduled time is now or in the past, and the run was triggered before
-                if seconds <= 0 and data['_when_run']:
-                    data['_when_run'] = False
-                    run = True
-            elif 'cron' in data:
-                # Run the job only if less than or just one second left till
-                # the scheduled time, or during the next loop interval right after that.
-                if data['_cron_next'] <= 1 or \
-                    (data['_cron_prev'] < self.option('loop_interval') and
-                        data['_cron_prev'] < self.loop_interval):
-                    run = True
-            else:
-                if job in self.intervals:
-                    if now - self.intervals[job] >= seconds:
-                        run = True
-                else:
-                    # If run_on_start is True, the job will run when the Salt
-                    # minion start.  If the value is False will run at the next
-                    # scheduled run.  Default is True.
-                    if 'run_on_start' in data:
-                        if data['run_on_start']:
-                            run = True
-                        else:
-                            self.intervals[job] = int(time.time())
+                # Got "splay" configured, make decision to run a job based on that
+                if not data['_splay']:
+                    # Try to add "splay" time only if next job fire time is
+                    # still in the future. We should trigger job run
+                    # immediately otherwise.
+                    splay = _splay(data['splay'])
+                    if now < data['_next_fire_time'] + splay:
+                        log.debug('schedule.handle_func: Adding splay of '
+                                  '{0} seconds to next run.'.format(splay))
+                        data['_splay'] = data['_next_fire_time'] + splay
+                        if 'when' in data:
+                            data['_run'] = True
                     else:
                         run = True
 
-            if run:
+                if data['_splay']:
+                    # The "splay" configuration has been already processed, just use it
+                    seconds = data['_splay'] - now
+
+            if seconds <= 0:
+                if '_seconds' in data:
+                    run = True
+                elif 'when' in data and data['_run']:
+                    data['_run'] = False
+                    run = True
+                elif 'cron' in data:
+                    # Reset next scheduled time because it is in the past now,
+                    # and we should trigger the job run, then wait for the next one.
+                    data['_next_fire_time'] = None
+                    run = True
+                elif seconds == 0:
+                    run = True
+
+            if '_run_on_start' in data and data['_run_on_start']:
+                run = True
+                data['_run_on_start'] = False
+            elif run:
                 if 'range' in data:
                     if not _RANGE_SUPPORTED:
                         log.error('Missing python-dateutil. Ignoring job {0}'.format(job))
@@ -1298,7 +1351,7 @@ class Schedule(object):
                                     else:
                                         run = False
                                 else:
-                                    if now >= start and now <= end:
+                                    if start <= now <= end:
                                         run = True
                                     else:
                                         run = False
@@ -1313,30 +1366,13 @@ class Schedule(object):
 
             if not run:
                 continue
-            else:
-                if 'splay' in data:
-                    if 'when' in data:
-                        log.error('Unable to use "splay" with "when" option at this time. Ignoring.')
-                    else:
-                        if isinstance(data['splay'], dict):
-                            if data['splay']['end'] >= data['splay']['start']:
-                                splay = random.randint(data['splay']['start'], data['splay']['end'])
-                            else:
-                                log.error('schedule.handle_func: Invalid Splay, end must be larger than start. \
-                                         Ignoring splay.')
-                                splay = None
-                        else:
-                            splay = random.randint(0, data['splay'])
 
-                        if splay:
-                            log.debug('schedule.handle_func: Adding splay of '
-                                      '{0} seconds to next run.'.format(splay))
-                            if 'seconds' in data:
-                                data['seconds'] = data['_seconds'] + splay
-                            else:
-                                data['seconds'] = 0 + splay
+            miss_msg = ''
+            if seconds < 0:
+                miss_msg = ' (runtime missed ' \
+                           'by {0} seconds)'.format(abs(seconds))
 
-                log.info('Running scheduled job: {0}'.format(job))
+            log.info('Running scheduled job: {0}{1}'.format(job, miss_msg))
 
             if 'jid_include' not in data or data['jid_include']:
                 data['jid_include'] = True
@@ -1360,6 +1396,8 @@ class Schedule(object):
                 self.functions = {}
                 returners = self.returners
                 self.returners = {}
+                utils = self.utils
+                self.utils = {}
             try:
                 if multiprocessing_enabled:
                     thread_cls = SignalHandlingMultiprocessingProcess
@@ -1378,11 +1416,14 @@ class Schedule(object):
                 if multiprocessing_enabled:
                     proc.join()
             finally:
-                self.intervals[job] = now
+                if '_seconds' in data:
+                    data['_next_fire_time'] = now + data['_seconds']
+                data['_splay'] = None
             if salt.utils.is_windows():
                 # Restore our function references.
                 self.functions = functions
                 self.returners = returners
+                self.utils = utils
 
 
 def clean_proc_dir(opts):
