@@ -16,9 +16,9 @@ import socket
 import logging
 
 # Import third party libs
-if sys.platform.startswith('win'):
+try:
     import win32file
-else:
+except ImportError:
     import resource
 
 # Import salt libs
@@ -28,9 +28,12 @@ from salt.exceptions import SaltClientError, SaltSystemExit, \
     CommandExecutionError
 import salt.defaults.exitcodes
 import salt.utils
+import salt.utils.files
 
 log = logging.getLogger(__name__)
 
+ROOT_DIR = 'c:\\salt' if salt.utils.is_windows() else '/'
+
 
 def zmq_version():
     '''
@@ -160,11 +163,12 @@ def verify_files(files, user):
     for fn_ in files:
         dirname = os.path.dirname(fn_)
         try:
-            try:
-                os.makedirs(dirname)
-            except OSError as err:
-                if err.errno != errno.EEXIST:
-                    raise
+            if dirname:
+                try:
+                    os.makedirs(dirname)
+                except OSError as err:
+                    if err.errno != errno.EEXIST:
+                        raise
             if not os.path.isfile(fn_):
                 with salt.utils.fopen(fn_, 'w+') as fp_:
                     fp_.write('')
@@ -191,15 +195,13 @@ def verify_files(files, user):
     return True
 
 
-def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
+def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False, root_dir=ROOT_DIR):
     '''
     Verify that the named directories are in place and that the environment
     can shake the salt
     '''
     if salt.utils.is_windows():
-        from salt.utils.win_functions import get_current_user
-        return win_verify_env(
-            dirs, get_current_user(), permissive, pki_dir, skip_extra)
+        return win_verify_env(root_dir, dirs, permissive, pki_dir, skip_extra)
     import pwd  # after confirming not running Windows
     try:
         pwnam = pwd.getpwnam(user)
@@ -217,12 +219,11 @@ def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
             continue
         if not os.path.isdir(dir_):
             try:
-                cumask = os.umask(18)  # 077
-                os.makedirs(dir_)
+                with salt.utils.files.set_umask(0o022):
+                    os.makedirs(dir_)
                 # If starting the process as root, chown the new dirs
                 if os.getuid() == 0:
                     os.chown(dir_, uid, gid)
-                os.umask(cumask)
             except OSError as err:
                 msg = 'Failed to create directory path "{0}" - {1}\n'
                 sys.stderr.write(msg.format(dir_, err))
@@ -518,23 +519,34 @@ def verify_log(opts):
     '''
     If an insecre logging configuration is found, show a warning
     '''
-    level = LOG_LEVELS.get(opts.get('log_level').lower(), logging.NOTSET)
+    level = LOG_LEVELS.get(str(opts.get('log_level')).lower(), logging.NOTSET)
 
     if level < logging.INFO:
         log.warning('Insecure logging configuration detected! Sensitive data may be logged.')
 
 
-def win_verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
+def win_verify_env(path, dirs, permissive=False, pki_dir='', skip_extra=False):
     '''
     Verify that the named directories are in place and that the environment
     can shake the salt
     '''
     import salt.utils.win_functions
+    import salt.utils.win_dacl
+    import salt.utils.path
 
-    # Get the root path directory where salt is installed
-    path = dirs[0]
-    while os.path.basename(path) not in ['salt', 'salt-tests-tmpdir']:
-        path, base = os.path.split(path)
+    # Make sure the file_roots is not set to something unsafe since permissions
+    # on that directory are reset
+
+    # `salt.utils.path.safe_path` will consider anything inside `C:\Windows` to
+    # be unsafe. In some instances the test suite uses
+    # `C:\Windows\Temp\salt-tests-tmpdir\rootdir` as the file_roots. So, we need
+    # to consider anything in `C:\Windows\Temp` to be safe
+    system_root = os.environ.get('SystemRoot', r'C:\Windows')
+    allow_path = '\\'.join([system_root, 'TEMP'])
+    if not salt.utils.path.safe_path(path=path, allow_path=allow_path):
+        raise CommandExecutionError(
+            '`file_roots` set to a possibly unsafe location: {0}'.format(path)
+        )
 
     # Create the root path directory if missing
     if not os.path.isdir(path):
@@ -544,10 +556,12 @@ def win_verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
     current_user = salt.utils.win_functions.get_current_user()
     if salt.utils.win_functions.is_admin(current_user):
         try:
-            salt.utils.win_functions.set_path_owner(path)
+            # Make the Administrators group owner
+            # Use the SID to be locale agnostic
+            salt.utils.win_dacl.set_owner(path, 'S-1-5-32-544')
+
         except CommandExecutionError:
-            msg = 'Unable to securely set the owner of "{0}".'
-            msg = msg.format(path)
+            msg = 'Unable to securely set the owner of "{0}".'.format(path)
             if is_console_configured():
                 log.critical(msg)
             else:
@@ -555,10 +569,26 @@ def win_verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
 
         if not permissive:
             try:
-                salt.utils.win_functions.set_path_permissions(path)
+                # Get a clean dacl by not passing an obj_name
+                dacl = salt.utils.win_dacl.dacl()
+
+                # Add aces to the dacl, use the GUID (locale non-specific)
+                # Administrators Group
+                dacl.add_ace('S-1-5-32-544', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # System
+                dacl.add_ace('S-1-5-18', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # Owner
+                dacl.add_ace('S-1-3-4', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+
+                # Save the dacl to the object
+                dacl.save(path, True)
+
             except CommandExecutionError:
-                msg = 'Unable to securely set the permissions of "{0}".'
-                msg = msg.format(path)
+                msg = 'Unable to securely set the permissions of ' \
+                      '"{0}".'.format(path)
                 if is_console_configured():
                     log.critical(msg)
                 else:
@@ -576,10 +606,30 @@ def win_verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
                 sys.stderr.write(msg.format(dir_, err))
                 sys.exit(err.errno)
 
+        # The PKI dir gets its own permissions
         if dir_ == pki_dir:
             try:
-                salt.utils.win_functions.set_path_owner(dir_)
-                salt.utils.win_functions.set_path_permissions(dir_)
+                # Make Administrators group the owner
+                salt.utils.win_dacl.set_owner(path, 'S-1-5-32-544')
+
+                # Give Admins, System and Owner permissions
+                # Get a clean dacl by not passing an obj_name
+                dacl = salt.utils.win_dacl.dacl()
+
+                # Add aces to the dacl, use the GUID (locale non-specific)
+                # Administrators Group
+                dacl.add_ace('S-1-5-32-544', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # System
+                dacl.add_ace('S-1-5-18', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # Owner
+                dacl.add_ace('S-1-3-4', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+
+                # Save the dacl to the object
+                dacl.save(dir_, True)
+
             except CommandExecutionError:
                 msg = 'Unable to securely set the permissions of "{0}".'
                 msg = msg.format(dir_)
